@@ -68,7 +68,7 @@ extern "C" {
 
 /* Headers in which EVERYONE has an interest... */
 
-#include "conf.h"
+#include "ap_config.h"
 #include "alloc.h"
 #include "buff.h"
 #include "ap.h"
@@ -79,7 +79,7 @@ extern "C" {
  * file with a relative pathname will have this added.
  */
 #ifndef HTTPD_ROOT
-#ifdef __EMX__
+#ifdef OS2
 /* Set default for OS/2 file system */
 #define HTTPD_ROOT "/os2httpd"
 #elif defined(WIN32)
@@ -94,7 +94,7 @@ extern "C" {
  * directive.
  */
 #ifndef DOCUMENT_LOCATION
-#ifdef __EMX__
+#ifdef OS2
 /* Set default for OS/2 file system */
 #define DOCUMENT_LOCATION  HTTPD_ROOT "/docs"
 #else
@@ -122,6 +122,8 @@ extern "C" {
 /* -- Internal representation for a HTTP protocol number, e.g., HTTP/1.1 -- */
 
 #define HTTP_VERSION(major,minor) (1000*(major)+(minor))
+#define HTTP_VERSION_MAJOR(number) ((number)/1000)
+#define HTTP_VERSION_MINOR(number) ((number)%1000)
 
 
 /* -------------- Port number for server running standalone --------------- */
@@ -144,7 +146,7 @@ extern "C" {
 
 /* The name of the log files */
 #ifndef DEFAULT_XFERLOG
-#ifdef __EMX__
+#ifdef OS2
 /* Set default for OS/2 file system */
 #define DEFAULT_XFERLOG "logs/access.log"
 #else
@@ -153,7 +155,7 @@ extern "C" {
 #endif /* DEFAULT_XFERLOG */
 
 #ifndef DEFAULT_ERRORLOG
-#ifdef __EMX__
+#ifdef OS2
 /* Set default for OS/2 file system */
 #define DEFAULT_ERRORLOG "logs/error.log"
 #else
@@ -189,7 +191,7 @@ extern "C" {
 
 /* Define this to be what your per-directory security files are called */
 #ifndef DEFAULT_ACCESS_FNAME
-#ifdef __EMX__
+#ifdef OS2
 /* Set default for OS/2 file system */
 #define DEFAULT_ACCESS_FNAME "htaccess"
 #else
@@ -233,7 +235,7 @@ extern "C" {
 
 /* The path to the shell interpreter, for parsed docs */
 #ifndef SHELL_PATH
-#if defined(__EMX__) || defined(WIN32)
+#if defined(OS2) || defined(WIN32)
 /* Set default for OS/2 and Windows file system */
 #define SHELL_PATH "CMD.EXE"
 #else
@@ -369,6 +371,32 @@ extern "C" {
 #define DEFAULT_LISTENBACKLOG 511
 #endif
 
+/* Limits on the size of various request items.  These limits primarily
+ * exist to prevent simple denial-of-service attacks on a server based
+ * on misuse of the protocol.  The recommended values will depend on the
+ * nature of the server resources -- CGI scripts and database backends
+ * might require large values, but most servers could get by with much
+ * smaller limits than we use below.  The request message body size can
+ * be limited by the per-dir config directive LimitRequestBody.
+ *
+ * Internal buffer sizes are two bytes more than the DEFAULT_LIMIT_REQUEST_LINE
+ * and DEFAULT_LIMIT_REQUEST_FIELDSIZE below, which explains the 8190.
+ * These two limits can be lowered (but not raised) by the server config
+ * directives LimitRequestLine and LimitRequestFieldsize, respectively.
+ *
+ * DEFAULT_LIMIT_REQUEST_FIELDS can be modified or disabled (set = 0) by
+ * the server config directive LimitRequestFields.
+ */
+#ifndef DEFAULT_LIMIT_REQUEST_LINE
+#define DEFAULT_LIMIT_REQUEST_LINE 8190
+#endif /* default limit on bytes in Request-Line (Method+URI+HTTP-version) */
+#ifndef DEFAULT_LIMIT_REQUEST_FIELDSIZE
+#define DEFAULT_LIMIT_REQUEST_FIELDSIZE 8190
+#endif /* default limit on bytes in any one header field  */
+#ifndef DEFAULT_LIMIT_REQUEST_FIELDS
+#define DEFAULT_LIMIT_REQUEST_FIELDS 100
+#endif /* default limit on number of request header fields */
+
 /*
  * The below defines the base string of the Server: header. Additional
  * tokens can be added via the ap_add_version_component() API call.
@@ -382,7 +410,7 @@ extern "C" {
  * Example: "Apache/1.1.0 MrWidget/0.1-alpha" 
  */
 
-#define SERVER_BASEVERSION "Apache/1.3.0"	/* SEE COMMENTS ABOVE */
+#define SERVER_BASEVERSION "Apache/1.3.3"	/* SEE COMMENTS ABOVE */
 #define SERVER_VERSION  SERVER_BASEVERSION
 enum server_token_type {
     SrvTk_MIN,		/* eg: Apache/1.3.0 */
@@ -396,8 +424,10 @@ API_EXPORT(const char *) ap_get_server_built(void);
 
 /* Numeric release version identifier: major minor bugfix betaseq
  * Always increases along the same track as the source branch.
+ * For a final release, 'betaseq' should be set to '99'.
+ * For example, Apache 1.4.2 should be '1040299'
  */
-#define APACHE_RELEASE 1030000
+#define APACHE_RELEASE 1030399
 
 #define SERVER_PROTOCOL "HTTP/1.1"
 #ifndef SERVER_SUPPORT
@@ -413,10 +443,15 @@ API_EXPORT(const char *) ap_get_server_built(void);
 
 /* ----------------------- HTTP Status Codes  ------------------------- */
 
-#define RESPONSE_CODES 38
+/* The size of the static array in http_protocol.c for storing
+ * all of the potential response status-lines (a sparse table).
+ * A future version should dynamically generate the table at startup.
+ */
+#define RESPONSE_CODES 54
 
 #define HTTP_CONTINUE                      100
 #define HTTP_SWITCHING_PROTOCOLS           101
+#define HTTP_PROCESSING                    102
 #define HTTP_OK                            200
 #define HTTP_CREATED                       201
 #define HTTP_ACCEPTED                      202
@@ -424,12 +459,14 @@ API_EXPORT(const char *) ap_get_server_built(void);
 #define HTTP_NO_CONTENT                    204
 #define HTTP_RESET_CONTENT                 205
 #define HTTP_PARTIAL_CONTENT               206
+#define HTTP_MULTI_STATUS                  207
 #define HTTP_MULTIPLE_CHOICES              300
 #define HTTP_MOVED_PERMANENTLY             301
 #define HTTP_MOVED_TEMPORARILY             302
 #define HTTP_SEE_OTHER                     303
 #define HTTP_NOT_MODIFIED                  304
 #define HTTP_USE_PROXY                     305
+#define HTTP_TEMPORARY_REDIRECT            307
 #define HTTP_BAD_REQUEST                   400
 #define HTTP_UNAUTHORIZED                  401
 #define HTTP_PAYMENT_REQUIRED              402
@@ -446,6 +483,10 @@ API_EXPORT(const char *) ap_get_server_built(void);
 #define HTTP_REQUEST_ENTITY_TOO_LARGE      413
 #define HTTP_REQUEST_URI_TOO_LARGE         414
 #define HTTP_UNSUPPORTED_MEDIA_TYPE        415
+#define HTTP_RANGE_NOT_SATISFIABLE         416
+#define HTTP_EXPECTATION_FAILED            417
+#define HTTP_UNPROCESSABLE_ENTITY          422
+#define HTTP_LOCKED                        423
 #define HTTP_INTERNAL_SERVER_ERROR         500
 #define HTTP_NOT_IMPLEMENTED               501
 #define HTTP_BAD_GATEWAY                   502
@@ -453,6 +494,7 @@ API_EXPORT(const char *) ap_get_server_built(void);
 #define HTTP_GATEWAY_TIME_OUT              504
 #define HTTP_VERSION_NOT_SUPPORTED         505
 #define HTTP_VARIANT_ALSO_VARIES           506
+#define HTTP_NOT_EXTENDED                  510
 
 #define DOCUMENT_FOLLOWS    HTTP_OK
 #define PARTIAL_CONTENT     HTTP_PARTIAL_CONTENT
@@ -473,12 +515,12 @@ API_EXPORT(const char *) ap_get_server_built(void);
 #define BAD_GATEWAY         HTTP_BAD_GATEWAY
 #define VARIANT_ALSO_VARIES HTTP_VARIANT_ALSO_VARIES
 
-#define is_HTTP_INFO(x)         (((x) >= 100)&&((x) < 200))
-#define is_HTTP_SUCCESS(x)      (((x) >= 200)&&((x) < 300))
-#define is_HTTP_REDIRECT(x)     (((x) >= 300)&&((x) < 400))
-#define is_HTTP_ERROR(x)        (((x) >= 400)&&((x) < 600))
-#define is_HTTP_CLIENT_ERROR(x) (((x) >= 400)&&((x) < 500))
-#define is_HTTP_SERVER_ERROR(x) (((x) >= 500)&&((x) < 600))
+#define ap_is_HTTP_INFO(x)         (((x) >= 100)&&((x) < 200))
+#define ap_is_HTTP_SUCCESS(x)      (((x) >= 200)&&((x) < 300))
+#define ap_is_HTTP_REDIRECT(x)     (((x) >= 300)&&((x) < 400))
+#define ap_is_HTTP_ERROR(x)        (((x) >= 400)&&((x) < 600))
+#define ap_is_HTTP_CLIENT_ERROR(x) (((x) >= 400)&&((x) < 500))
+#define ap_is_HTTP_SERVER_ERROR(x) (((x) >= 500)&&((x) < 600))
 
 #define status_drops_connection(x) (((x) == HTTP_BAD_REQUEST)           || \
                                     ((x) == HTTP_REQUEST_TIME_OUT)      || \
@@ -598,7 +640,7 @@ struct request_rec {
     int header_only;		/* HEAD request, as opposed to GET */
     char *protocol;		/* Protocol, as given to us, or HTTP/0.9 */
     int proto_num;		/* Number version of protocol; 1.1 = 1001 */
-    char *hostname;		/* Host, as set by full URI or Host: */
+    const char *hostname;	/* Host, as set by full URI or Host: */
 
     time_t request_time;	/* When the request started */
 
@@ -643,7 +685,7 @@ struct request_rec {
     int chunked;		/* sending chunked transfer-coding */
     int byterange;		/* number of byte ranges */
     char *boundary;		/* multipart/byteranges boundary */
-    char *range;		/* The Range: header */
+    const char *range;		/* The Range: header */
     long clength;		/* The "real" content length */
 
     long remaining;		/* bytes left to read */
@@ -673,11 +715,11 @@ struct request_rec {
      * content_languages MUST be lowercased strings.  They may be pointers
      * to static strings; they should not be modified in place.
      */
-    char *content_type;		/* Break these out --- we dispatch on 'em */
-    char *handler;		/* What we *really* dispatch on           */
+    const char *content_type;	/* Break these out --- we dispatch on 'em */
+    const char *handler;	/* What we *really* dispatch on           */
 
-    char *content_encoding;
-    char *content_language;	/* for back-compat. only -- do not use */
+    const char *content_encoding;
+    const char *content_language;	/* for back-compat. only -- do not use */
     array_header *content_languages;	/* array of (char*) */
 
     int no_cache;
@@ -710,6 +752,13 @@ struct request_rec {
  * that way, a sub request's list can (temporarily) point to a parent's list
  */
     const struct htaccess_result *htaccess;
+
+/* Things placed at the end of the record to avoid breaking binary
+ * compatibility.  It would be nice to remember to reorder the entire
+ * record to improve 64bit alignment the next time we need to break
+ * binary compatibility for some other reason.
+ */
+    unsigned expecting_100;     /* is client waiting for a 100 response? */
 };
 
 
@@ -820,8 +869,12 @@ struct server_rec {
     array_header *names;	/* Normal names for ServerAlias servers */
     array_header *wild_names;	/* Wildcarded names for ServerAlias servers */
 
-    uid_t server_uid;		/* effective user id when calling exec wrapper */
-    gid_t server_gid;		/* effective group id when calling exec wrapper */
+    uid_t server_uid;        /* effective user id when calling exec wrapper */
+    gid_t server_gid;        /* effective group id when calling exec wrapper */
+
+    int limit_req_line;      /* limit on size of the HTTP request line    */
+    int limit_req_fieldsize; /* limit on size of any request header field */
+    int limit_req_fields;    /* limit on number of request header fields  */
 };
 
 /* These are more like real hosts than virtual hosts */
@@ -860,7 +913,7 @@ API_EXPORT(char *) ap_getword_nulls_nc(pool *p, char **line, char stop);
 API_EXPORT(char *) ap_getword_conf(pool *p, const char **line);
 API_EXPORT(char *) ap_getword_conf_nc(pool *p, char **line);
 
-API_EXPORT(char *) ap_get_token(pool *p, char **accept_line, int accept_white);
+API_EXPORT(char *) ap_get_token(pool *p, const char **accept_line, int accept_white);
 API_EXPORT(int) ap_find_token(pool *p, const char *line, const char *tok);
 API_EXPORT(int) ap_find_last_token(pool *p, const char *line, const char *tok);
 
@@ -887,13 +940,15 @@ API_EXPORT(int) ap_is_matchexp(const char *str);
 API_EXPORT(int) ap_strcmp_match(const char *str, const char *exp);
 API_EXPORT(int) ap_strcasecmp_match(const char *str, const char *exp);
 API_EXPORT(char *) ap_uudecode(pool *, const char *);
-#ifdef __EMX__
+API_EXPORT(char *) ap_uuencode(pool *p, char *string); 
+#ifdef OS2
 void os2pathname(char *path);
 #endif
 
 API_EXPORT(char *) ap_pregsub(pool *p, const char *input, const char *source,
 			   size_t nmatch, regmatch_t pmatch[]);
 
+API_EXPORT(void) ap_content_type_tolower(char *);
 API_EXPORT(void) ap_str_tolower(char *);
 API_EXPORT(int) ap_ind(const char *, char);	/* Sigh... */
 API_EXPORT(int) ap_rind(const char *, char);
@@ -948,12 +1003,12 @@ API_EXPORT(char *) ap_os_canonical_filename(pool *p, const char *file);
 #endif
 
 #ifdef _OSD_POSIX
-extern const char *os_set_authfile(pool *p, const char *filename);
+extern const char *os_set_account(pool *p, const char *account);
 extern int os_init_job_environment(server_rec *s, const char *user_name);
 #endif /* _OSD_POSIX */
 
 char *ap_get_local_host(pool *);
-unsigned long ap_get_virthost_addr(const char *hostname, unsigned short *port);
+unsigned long ap_get_virthost_addr(char *hostname, unsigned short *port);
 
 extern API_VAR_EXPORT time_t ap_restart_time;
 
@@ -989,6 +1044,8 @@ int ap_slack(int fd, int line);
 #define AP_SLACK_HIGH	2
 #endif
 
+API_EXPORT(char *) ap_escape_quotes(pool *p, const char *instr);
+
 /*
  * Redefine assert() to something more useful for an Apache...
  */
