@@ -476,8 +476,10 @@ static void init_handlers(pool *p)
 int ap_invoke_handler(request_rec *r)
 {
     fast_handler_rec *handp;
-    char *handler, *p;
+    const char *handler;
+    char *p;
     size_t handler_len;
+    int result = NOT_IMPLEMENTED;
 
     if (r->handler) {
 	handler = r->handler;
@@ -500,19 +502,24 @@ int ap_invoke_handler(request_rec *r)
     for (handp = handlers; handp->hr.content_type; ++handp) {
 	if (handler_len == handp->len
 	    && !strncmp(handler, handp->hr.content_type, handler_len)) {
-            int result = (*handp->hr.handler) (r);
+            result = (*handp->hr.handler) (r);
 
             if (result != DECLINED)
                 return result;
         }
     }
 
+    if (result == NOT_IMPLEMENTED && r->handler) {
+        ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_WARNING, r,
+            "handler \"%s\" not found for: %s", r->handler, r->filename);
+    }
+
     /* Pass two --- wildcard matches */
 
     for (handp = wildhandlers; handp->hr.content_type; ++handp) {
 	if (handler_len >= handp->len
 	    && !strncmp(handler, handp->hr.content_type, handp->len)) {
-             int result = (*handp->hr.handler) (r);
+             result = (*handp->hr.handler) (r);
 
              if (result != DECLINED)
                  return result;
@@ -531,15 +538,10 @@ API_EXPORT(void) ap_add_module(module *m)
      * teased out...
      */
 
-    /* At some point, we may want to offer back-compatibility for
-     * loading modules that are for older versions of Apache. For now,
-     * though, we don't.
-     */
-
-    if (m->version != MODULE_MAGIC_NUMBER) {
+    if (m->version != MODULE_MAGIC_NUMBER_MAJOR) {
 	fprintf(stderr, "httpd: module \"%s\" is not compatible with this "
 		"version of Apache.\n", m->name);
-	fprintf(stderr, "Please contact the author for the correct version.\n");
+	fprintf(stderr, "Please contact the vendor for the correct version.\n");
 	exit(1);
     }
 
@@ -596,6 +598,7 @@ API_EXPORT(void) ap_remove_module(module *m)
     if (modp == m) {
 	/* We are the top module, special case */
 	top_module = modp->next;
+	m->next = NULL;
     }
     else {
 	/* Not the top module, find use. When found modp will
@@ -1203,7 +1206,7 @@ int ap_parse_htaccess(void **result, request_rec *r, int override,
     char *filename = NULL;
     const struct htaccess_result *cache;
     struct htaccess_result *new;
-    void *dc;
+    void *dc = NULL;
 
 /* firstly, search cache */
     for (cache = r->htaccess; cache != NULL; cache = cache->next)
@@ -1221,37 +1224,39 @@ int ap_parse_htaccess(void **result, request_rec *r, int override,
     parms.path = ap_pstrdup(r->pool, d);
 
     /* loop through the access names and find the first one */
-    while (!f && access_name[0]) {
-	char *w = ap_getword_conf(r->pool, &access_name);
-	filename = ap_make_full_path(r->pool, d, w);
-	f = ap_pcfg_openfile(r->pool, filename);
-    }
-    if (f) {
-	dc = ap_create_per_dir_config(r->pool);
 
-	parms.config_file = f;
+    while (access_name[0]) {
+        filename = ap_make_full_path(r->pool, d,
+                                     ap_getword_conf(r->pool, &access_name));
 
-	errmsg = ap_srm_command_loop(&parms, dc);
+        if ((f = ap_pcfg_openfile(r->pool, filename)) != NULL) {
 
-	ap_cfg_closefile(f);
+            dc = ap_create_per_dir_config(r->pool);
 
-	if (errmsg) {
-	    ap_log_error(APLOG_MARK, APLOG_ALERT|APLOG_NOERRNO, r->server, "%s: %s",
-                        filename, errmsg);
-            return HTTP_INTERNAL_SERVER_ERROR;
-	}
+            parms.config_file = f;
 
-	*result = dc;
-    }
-    else {
-	if (errno == ENOENT || errno == ENOTDIR)
-	    dc = NULL;
-	else {
-	    ap_log_error(APLOG_MARK, APLOG_CRIT, r->server,
-			"%s pcfg_openfile: unable to check htaccess file, ensure it is readable",
-			filename);
-	    return HTTP_FORBIDDEN;
-	}
+            errmsg = ap_srm_command_loop(&parms, dc);
+
+            ap_cfg_closefile(f);
+
+            if (errmsg) {
+                ap_log_rerror(APLOG_MARK, APLOG_ALERT|APLOG_NOERRNO, r,
+                              "%s: %s", filename, errmsg);
+                return HTTP_INTERNAL_SERVER_ERROR;
+            }
+            *result = dc;
+            break;
+        }
+        else if (errno != ENOENT && errno != ENOTDIR) {
+            ap_log_rerror(APLOG_MARK, APLOG_CRIT, r,
+                          "%s pcfg_openfile: unable to check htaccess file, "
+                          "ensure it is readable",
+                          filename);
+            ap_table_setn(r->notes, "error-notes",
+                          "Server unable to read htaccess file, denying "
+                          "access to be safe");
+            return HTTP_FORBIDDEN;
+        }
     }
 
 /* cache it */
@@ -1310,6 +1315,10 @@ CORE_EXPORT(const char *) ap_init_virtual_host(pool *p, const char *hostname,
     s->server_uid = ap_user_id;
     s->server_gid = ap_group_id;
 
+    s->limit_req_line = main_server->limit_req_line;
+    s->limit_req_fieldsize = main_server->limit_req_fieldsize;
+    s->limit_req_fields = main_server->limit_req_fields;
+
     *ps = s;
 
     return ap_parse_vhost_addrs(p, hostname, s);
@@ -1383,6 +1392,7 @@ static void init_config_globals(pool *p)
     ap_bind_address.s_addr = htonl(INADDR_ANY);
     ap_listeners = NULL;
     ap_listenbacklog = DEFAULT_LISTENBACKLOG;
+    ap_extended_status = 0;
 
     /* Global virtual host hash bucket pointers.  Init to null. */
     ap_init_vhost_config(p);
@@ -1402,6 +1412,9 @@ static server_rec *init_server_config(pool *p)
     s->loglevel = DEFAULT_LOGLEVEL;
     s->srm_confname = RESOURCE_CONFIG_FILE;
     s->access_confname = ACCESS_CONFIG_FILE;
+    s->limit_req_line = DEFAULT_LIMIT_REQUEST_LINE;
+    s->limit_req_fieldsize = DEFAULT_LIMIT_REQUEST_FIELDSIZE;
+    s->limit_req_fields = DEFAULT_LIMIT_REQUEST_FIELDS;
     s->timeout = DEFAULT_TIMEOUT;
     s->keep_alive_timeout = DEFAULT_KEEPALIVE_TIMEOUT;
     s->keep_alive_max = DEFAULT_KEEPALIVE;
