@@ -71,6 +71,7 @@
 #include "http_request.h"
 #include "http_core.h"
 #include "http_protocol.h"
+#include "http_conf_globals.h"	/* for ap_extended_status */
 #include "http_log.h"
 #include "http_main.h"
 #include "scoreboard.h"
@@ -103,7 +104,7 @@ static int check_safe_file(request_rec *r)
         || S_ISLNK(r->finfo.st_mode)) {
         return OK;
     }
-    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
                 "object is not a file, directory or symlink: %s",
                 r->filename);
     return HTTP_FORBIDDEN;
@@ -112,7 +113,7 @@ static int check_safe_file(request_rec *r)
 
 static int check_symlinks(char *d, int opts)
 {
-#if defined(__EMX__) || defined(WIN32)
+#if defined(OS2) || defined(WIN32)
     /* OS/2 doesn't have symlinks */
     return OK;
 #else
@@ -187,8 +188,10 @@ static int get_path_info(request_rec *r)
     }
 
 #ifdef WIN32
-    /* If the path is x:/, then convert it to x:/., coz that's what stat needs to work properly */
-    if(strlen(path) == 3 && path[1] == ':') {
+    /* If the path is x:/, then convert it to x:/., coz that's what stat
+     * needs to work properly
+     */
+    if (strlen(path) == 3 && path[1] == ':') {
 	strcpy(buf,path);
 	buf[3]='.';
 	buf[4]='\0';
@@ -246,10 +249,8 @@ static int get_path_info(request_rec *r)
 #if defined(EACCES)
             if (errno != EACCES)
 #endif
-                ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-                            "access to %s failed for %s", r->uri,
-                            ap_get_remote_host(r->connection, r->per_dir_config,
-                                            REMOTE_NOLOOKUP));
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
+                            "access to %s failed", r->uri);
             return HTTP_FORBIDDEN;
         }
 #else
@@ -408,7 +409,7 @@ static int directory_walk(request_rec *r)
          */
 
         if ((res = check_symlinks(test_dirname, core_dir->opts))) {
-            ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
                         "Symbolic link not allowed: %s", test_dirname);
             return res;
         }
@@ -503,7 +504,7 @@ static int directory_walk(request_rec *r)
      */
     if (!S_ISDIR(r->finfo.st_mode)
         && (res = check_symlinks(r->filename, ap_allow_options(r)))) {
-        ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
                     "Symbolic link not allowed: %s", r->filename);
         return res;
     }
@@ -675,6 +676,7 @@ API_EXPORT(request_rec *) ap_sub_req_lookup_uri(const char *new_file,
     char *udir;
 
     rnew = make_sub_request(r);
+    rnew->hostname       = r->hostname;
     rnew->request_time   = r->request_time;
     rnew->connection     = r->connection;
     rnew->server         = r->server;
@@ -752,6 +754,7 @@ API_EXPORT(request_rec *) ap_sub_req_lookup_file(const char *new_file,
     char *fdir;
 
     rnew = make_sub_request(r);
+    rnew->hostname       = r->hostname;
     rnew->request_time   = r->request_time;
     rnew->connection     = r->connection;
     rnew->server         = r->server;
@@ -797,7 +800,7 @@ API_EXPORT(request_rec *) ap_sub_req_lookup_file(const char *new_file,
         }
         else {
             if ((res = check_symlinks(rnew->filename, ap_allow_options(rnew)))) {
-                ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, rnew->server,
+                ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, rnew,
                             "Symbolic link not allowed: %s", rnew->filename);
                 rnew->status = res;
                 return rnew;
@@ -827,7 +830,14 @@ API_EXPORT(request_rec *) ap_sub_req_lookup_file(const char *new_file,
          * file may not have a uri associated with it -djg
          */
         rnew->uri = "INTERNALLY GENERATED file-relative req";
+#ifdef WIN32
+        rnew->filename = ((new_file[0] == '/'
+                           || (ap_isalpha(new_file[0])
+                               && new_file[1] == ':'
+                               && new_file[2] == '/')) ?
+#else
         rnew->filename = ((new_file[0] == '/') ?
+#endif
                           ap_pstrdup(rnew->pool, new_file) :
                           ap_make_full_path(rnew->pool, fdir, new_file));
         rnew->per_dir_config = r->server->lookup_defaults;
@@ -875,7 +885,7 @@ API_EXPORT(void) ap_destroy_sub_req(request_rec *r)
  * Mainline request processing...
  */
 
-void ap_die(int type, request_rec *r)
+API_EXPORT(void) ap_die(int type, request_rec *r)
 {
     int error_index = ap_index_of_response(type);
     char *custom_response = ap_response_code_string(r, error_index);
@@ -943,7 +953,7 @@ void ap_die(int type, request_rec *r)
             ap_table_setn(r->headers_out, "Location", custom_response);
         }
         else if (custom_response[0] == '/') {
-            char *error_notes;
+            const char *error_notes;
             r->no_local_copy = 1;       /* Do NOT send USE_LOCAL_COPY for
                                          * error documents! */
             /*
@@ -971,7 +981,7 @@ void ap_die(int type, request_rec *r)
              * dying with a recursive server error...
              */
             recursive_error = SERVER_ERROR;
-            ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
                         "Invalid error redirection directive: %s",
                         custom_response);
         }
@@ -982,7 +992,7 @@ void ap_die(int type, request_rec *r)
 static void decl_die(int status, char *phase, request_rec *r)
 {
     if (status == DECLINED) {
-        ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_CRIT, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_CRIT, r,
                     "configuration error:  couldn't %s: %s", phase, r->uri);
         ap_die(SERVER_ERROR, r);
     }
@@ -994,7 +1004,7 @@ API_EXPORT(int) ap_some_auth_required(request_rec *r)
 {
     /* Is there a require line configured for the type of *this* req? */
 
-    array_header *reqs_arr = ap_requires(r);
+    const array_header *reqs_arr = ap_requires(r);
     require_line *reqs;
     int i;
 
@@ -1014,39 +1024,6 @@ static void process_request_internal(request_rec *r)
 {
     int access_status;
 
-    /*
-     * Kluge to be reading the assbackwards field outside of protocol.c, but
-     * we've got to check for this sort of nonsense somewhere...
-     */
-
-    if (r->assbackwards && r->header_only) {
-        /*
-         * Client asked for headers only with HTTP/0.9, which doesn't send
-         * headers!  Have to dink things even to make sure the error message
-         * comes through...
-         */
-        ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
-                    "client sent illegal HTTP/0.9 request: %s", r->uri);
-        r->header_only = 0;
-        ap_die(BAD_REQUEST, r);
-        return;
-    }
-
-    if ((!r->hostname && (r->proto_num >= HTTP_VERSION(1,1))) ||
-        ((r->proto_num == HTTP_VERSION(1,1)) && !ap_table_get(r->headers_in, "Host"))) {
-        /*
-         * Client sent us a HTTP/1.1 or later request without telling us the
-         * hostname, either with a full URL or a Host: header. We therefore
-         * need to (as per the 1.1 spec) send an error.  As a special case,
-	 * HTTP/1.1 mentions twice (S9, S14.23) that a request MUST contain
-	 * a Host: header, and the server MUST respond with 400 if it doesn't.
-         */
-        ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
-               "client sent HTTP/1.1 request without hostname (see RFC2068 section 9, and 14.23): %s", r->uri);
-        ap_die(BAD_REQUEST, r);
-        return;
-    }
-
     /* Ignore embedded %2F's in path for proxy requests */
     if (!r->proxyreq && r->parsed_uri.path) {
 	access_status = ap_unescape_url(r->parsed_uri.path);
@@ -1182,18 +1159,15 @@ static void process_request_internal(request_rec *r)
 
 void ap_process_request(request_rec *r)
 {
-#ifdef STATUS
     int old_stat;
 
-    ap_time_process_request(r->connection->child_num, START_PREQUEST);
-#endif
+    if (ap_extended_status)
+	ap_time_process_request(r->connection->child_num, START_PREQUEST);
 
     process_request_internal(r);
 
-#ifdef STATUS
     old_stat = ap_update_child_status(r->connection->child_num,
                                    SERVER_BUSY_LOG, r);
-#endif
 
     /*
      * We want to flush the last packet if this isn't a pipelining connection
@@ -1205,15 +1179,14 @@ void ap_process_request(request_rec *r)
     ap_bhalfduplex(r->connection->client);
     ap_log_transaction(r);
 
-#ifdef STATUS
     (void) ap_update_child_status(r->connection->child_num, old_stat, r);
-    ap_time_process_request(r->connection->child_num, STOP_PREQUEST);
-#endif
+    if (ap_extended_status)
+	ap_time_process_request(r->connection->child_num, STOP_PREQUEST);
 }
 
 static table *rename_original_env(pool *p, table *t)
 {
-    array_header *env_arr = table_elts(t);
+    array_header *env_arr = ap_table_elts(t);
     table_entry *elts = (table_entry *) env_arr->elts;
     table *new = ap_make_table(p, env_arr->nalloc);
     int i;
