@@ -90,7 +90,7 @@
 
 #include <string.h>
 
-#include "conf.h"
+#include "ap_config.h"
 #include "ap_md5.h"
 #ifdef CHARSET_EBCDIC
 #include "ebcdic.h"
