@@ -72,7 +72,7 @@
 #include "http_conf_globals.h"	/* for user_id & group_id */
 #include "http_log.h"
 #if defined(SUNOS4)
-/* stdio.h has been read in conf.h already. Add missing prototypes here: */
+/* stdio.h has been read in ap_config.h already. Add missing prototypes here: */
 extern int fgetc(FILE *);
 extern char *fgets(char *s, int, FILE*);
 extern int fclose(FILE *);
@@ -257,7 +257,7 @@ API_EXPORT(int) ap_strcasecmp_match(const char *str, const char *exp)
 	    }
 	    return -1;
 	}
-	else if ((exp[y] != '?') && (tolower(str[x]) != tolower(exp[y])))
+	else if ((exp[y] != '?') && (ap_tolower(str[x]) != ap_tolower(exp[y])))
 	    return 1;
     }
     return (str[x] != '\0');
@@ -308,7 +308,7 @@ API_EXPORT(char *) ap_pregsub(pool *p, const char *input, const char *source,
     while ((c = *src++) != '\0') {
 	if (c == '&')
 	    no = 0;
-	else if (c == '$' && isdigit(*src))
+	else if (c == '$' && ap_isdigit(*src))
 	    no = *src++ - '0';
 	else
 	    no = 10;
@@ -333,7 +333,7 @@ API_EXPORT(char *) ap_pregsub(pool *p, const char *input, const char *source,
     while ((c = *src++) != '\0') {
 	if (c == '&')
 	    no = 0;
-	else if (c == '$' && isdigit(*src))
+	else if (c == '$' && ap_isdigit(*src))
 	    no = *src++ - '0';
 	else
 	    no = 10;
@@ -582,7 +582,7 @@ API_EXPORT(char *) ap_getword_white(pool *atrans, const char **line)
     char *res;
 
     for (x = 0; (*line)[x]; x++) {
-	if (isspace((*line)[x])) {
+	if (ap_isspace((*line)[x])) {
 	    pos = x;
 	    break;
 	}
@@ -597,7 +597,7 @@ API_EXPORT(char *) ap_getword_white(pool *atrans, const char **line)
     res = ap_palloc(atrans, pos + 1);
     ap_cpystrn(res, *line, pos + 1);
 
-    while (isspace((*line)[pos]))
+    while (ap_isspace((*line)[pos]))
 	++pos;
 
     *line += pos;
@@ -663,7 +663,7 @@ API_EXPORT(char *) ap_getword_conf(pool *p, const char **line)
     char *res;
     char quote;
 
-    while (*str && isspace(*str))
+    while (*str && ap_isspace(*str))
 	++str;
 
     if (!*str) {
@@ -686,13 +686,13 @@ API_EXPORT(char *) ap_getword_conf(pool *p, const char **line)
     }
     else {
 	strend = str;
-	while (*strend && !isspace(*strend))
+	while (*strend && !ap_isspace(*strend))
 	    ++strend;
 
 	res = substring_conf(p, str, strend - str, 0);
     }
 
-    while (*strend && isspace(*strend))
+    while (*strend && ap_isspace(*strend))
 	++strend;
     *line = strend;
     return res;
@@ -701,7 +701,8 @@ API_EXPORT(char *) ap_getword_conf(pool *p, const char **line)
 API_EXPORT(int) ap_cfg_closefile(configfile_t *cfp)
 {
 #ifdef DEBUG
-    ap_log_error(APLOG_MARK, APLOG_DEBUG|APLOG_NOERRNO, NULL, "Done with config file %s", fp->name);
+    ap_log_error(APLOG_MARK, APLOG_DEBUG|APLOG_NOERRNO, NULL, 
+        "Done with config file %s", cfp->name);
 #endif
     return (cfp->close == NULL) ? 0 : cfp->close(cfp->param);
 }
@@ -737,6 +738,7 @@ API_EXPORT(configfile_t *) ap_pcfg_openfile(pool *p, const char *name)
     poolfile_t *new_pfile;
     FILE *file;
     struct stat stbuf;
+    int saved_errno;
 
     if (name == NULL) {
         ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO, NULL,
@@ -746,9 +748,11 @@ API_EXPORT(configfile_t *) ap_pcfg_openfile(pool *p, const char *name)
 
     file = ap_pfopen(p, name, "r");
 #ifdef DEBUG
+    saved_errno = errno;
     ap_log_error(APLOG_MARK, APLOG_DEBUG | APLOG_NOERRNO, NULL,
                 "Opening config file %s (%s)",
                 name, (file == NULL) ? strerror(errno) : "successful");
+    errno = saved_errno;
 #endif
     if (file == NULL)
         return NULL;
@@ -760,10 +764,12 @@ API_EXPORT(configfile_t *) ap_pcfg_openfile(pool *p, const char *name)
 #else
         strcmp(name, "/dev/null") != 0) {
 #endif
+	saved_errno = errno;
         ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO, NULL,
                     "Access to file %s denied by server: not a regular file",
                     name);
         ap_pfclose(p, file);
+	errno = saved_errno;
         return NULL;
     }
 
@@ -863,28 +869,20 @@ API_EXPORT(int) ap_cfg_getline(char *buf, size_t bufsize, configfile_t *cfp)
 	    break;
 	}
 
-	/* Compress the line, reducing all blanks and tabs to one space.
+	/*
 	 * Leading and trailing white space is eliminated completely
 	 */
-	src = dst = buf;
-	while (isspace(*src))
+	src = buf;
+	while (ap_isspace(*src))
 	    ++src;
-	while (*src != '\0')
-	{
-	    /* Copy words */
-	    while (!isspace(*dst = *src) && *src != '\0') {
-		++src;
-		++dst;
-	    }
-	    if (*src == '\0') break;
-	    *dst++ = ' ';
-	    while (isspace(*src))
-		++src;
-	}
-	*dst = '\0';
 	/* blast trailing whitespace */
-	while (--dst >= buf && isspace(*dst))
+	dst = &src[strlen(src)];
+	while (--dst >= src && ap_isspace(*dst))
 	    *dst = '\0';
+        /* Zap leading whitespace by shifting */
+        if (src != buf)
+	    for (dst = buf; (*dst++ = *src++) != '\0'; )
+	        ;
 
 #ifdef DEBUG_CFG_LINES
 	ap_log_error(APLOG_MARK, APLOG_DEBUG|APLOG_NOERRNO, NULL, "Read config: %s", buf);
@@ -940,7 +938,7 @@ API_EXPORT(int) ap_cfg_getline(char *buf, size_t bufsize, configfile_t *cfp)
 		     */
 		}
 		/* blast trailing whitespace */
-		while (i > 0 && isspace(buf[i - 1]))
+		while (i > 0 && ap_isspace(buf[i - 1]))
 		    --i;
 		buf[i] = '\0';
 #ifdef DEBUG_CFG_LINES
@@ -961,16 +959,16 @@ API_EXPORT(int) ap_cfg_getline(char *buf, size_t bufsize, configfile_t *cfp)
  * by whitespace at the caller's option.
  */
 
-API_EXPORT(char *) ap_get_token(pool *p, char **accept_line, int accept_white)
+API_EXPORT(char *) ap_get_token(pool *p, const char **accept_line, int accept_white)
 {
-    char *ptr = *accept_line;
-    char *tok_start;
+    const char *ptr = *accept_line;
+    const char *tok_start;
     char *token;
     int tok_len;
 
     /* Find first non-white byte */
 
-    while (*ptr && isspace(*ptr))
+    while (*ptr && ap_isspace(*ptr))
 	++ptr;
 
     tok_start = ptr;
@@ -979,7 +977,7 @@ API_EXPORT(char *) ap_get_token(pool *p, char **accept_line, int accept_white)
      * (comments are already gone).
      */
 
-    while (*ptr && (accept_white || !isspace(*ptr))
+    while (*ptr && (accept_white || !ap_isspace(*ptr))
 	   && *ptr != ';' && *ptr != ',') {
 	if (*ptr++ == '"')
 	    while (*ptr)
@@ -992,7 +990,7 @@ API_EXPORT(char *) ap_get_token(pool *p, char **accept_line, int accept_white)
 
     /* Advance accept_line pointer to the next non-white byte */
 
-    while (*ptr && isspace(*ptr))
+    while (*ptr && ap_isspace(*ptr))
 	++ptr;
 
     *accept_line = ptr;
@@ -1047,7 +1045,7 @@ API_EXPORT(int) ap_find_last_token(pool *p, const char *line, const char *tok)
     lidx = llen - tlen;
 
     if ((lidx < 0) ||
-	((lidx > 0) && !(isspace(line[lidx - 1]) || line[lidx - 1] == ',')))
+	((lidx > 0) && !(ap_isspace(line[lidx - 1]) || line[lidx - 1] == ',')))
 	return 0;
 
     return (strncasecmp(&line[lidx], tok, tlen) == 0);
@@ -1064,7 +1062,7 @@ API_EXPORT(char *) ap_escape_shell_cmd(pool *p, const char *str)
     s = (const unsigned char *)str;
     for (; *s; ++s) {
 
-#if defined(__EMX__) || defined(WIN32)
+#if defined(OS2) || defined(WIN32)
 	/* Don't allow '&' in parameters under OS/2. */
 	/* This can be used to send commands to the shell. */
 	if (*s == '&') {
@@ -1309,7 +1307,7 @@ API_EXPORT(int) ap_is_url(const char *u)
 
     for (x = 0; u[x] != ':'; x++) {
 	if ((!u[x]) ||
-	    ((!isalpha(u[x])) && (!isdigit(u[x])) &&
+	    ((!ap_isalpha(u[x])) && (!ap_isdigit(u[x])) &&
 	     (u[x] != '+') && (u[x] != '-') && (u[x] != '.'))) {
 	    return 0;
 	}
@@ -1323,7 +1321,7 @@ API_EXPORT(int) ap_can_exec(const struct stat *finfo)
 #ifdef MULTIPLE_GROUPS
     int cnt;
 #endif
-#if defined(__EMX__) || defined(WIN32)
+#if defined(OS2) || defined(WIN32)
     /* OS/2 dosen't have Users and Groups */
     return 1;
 #else
@@ -1364,7 +1362,7 @@ int strcasecmp(const char *a, const char *b)
     const char *p = a;
     const char *q = b;
     for (p = a, q = b; *p && *q; p++, q++) {
-	int diff = tolower(*p) - tolower(*q);
+	int diff = ap_tolower(*p) - ap_tolower(*q);
 	if (diff)
 	    return diff;
     }
@@ -1389,7 +1387,7 @@ int strncasecmp(const char *a, const char *b, int n)
 	    return 0;		/*   Match up to n characters */
 	if (!(*p && *q))
 	    return *p - *q;
-	diff = tolower(*p) - tolower(*q);
+	diff = ap_tolower(*p) - ap_tolower(*q);
 	if (diff)
 	    return diff;
     }
@@ -1397,7 +1395,35 @@ int strncasecmp(const char *a, const char *b, int n)
 }
 #endif
 
-
+/* The following routine was donated for UTS21 by dwd@bell-labs.com */
+#ifdef NEED_STRSTR
+char *strstr(char *s1, char *s2)
+{
+    char *p1, *p2;
+    if (*s2 == '\0') {
+	/* an empty s2 */
+        return(s1);
+    }
+    while((s1 = strchr(s1, *s2)) != NULL) {
+	/* found first character of s2, see if the rest matches */
+        p1 = s1;
+        p2 = s2;
+        while (*++p1 == *++p2) {
+            if (*p1 == '\0') {
+                /* both strings ended together */
+                return(s1);
+            }
+        }
+        if (*p2 == '\0') {
+            /* second string ended, a match */
+            break;
+        }
+	/* didn't find a match here, try starting at next character in s1 */
+        s1++;
+    }
+    return(s1);
+}
+#endif
 
 #ifdef NEED_INITGROUPS
 int initgroups(const char *name, gid_t basegid)
@@ -1433,7 +1459,8 @@ int initgroups(const char *name, gid_t basegid)
 #ifdef NEED_WAITPID
 /* From ikluft@amdahl.com
  * this is not ideal but it works for SVR3 variants
- * httpd does not use the options so this doesn't implement them
+ * Modified by dwd@bell-labs.com to call wait3 instead of wait because
+ *   apache started to use the WNOHANG option.
  */
 int waitpid(pid_t pid, int *statusp, int options)
 {
@@ -1442,7 +1469,9 @@ int waitpid(pid_t pid, int *statusp, int options)
 	errno = ECHILD;
 	return -1;
     }
-    while (((tmp_pid = wait(statusp)) != pid) && (tmp_pid != -1));
+    while (((tmp_pid = wait3(statusp, options, 0)) != pid) &&
+		(tmp_pid != -1) && (tmp_pid != 0) && (pid != -1))
+	;
     return tmp_pid;
 }
 #endif
@@ -1472,7 +1501,7 @@ API_EXPORT(int) ap_rind(const char *s, char c)
 API_EXPORT(void) ap_str_tolower(char *str)
 {
     while (*str) {
-	*str = tolower(*str);
+	*str = ap_tolower(*str);
 	++str;
     }
 }
@@ -1518,7 +1547,7 @@ API_EXPORT(gid_t) ap_gname2id(const char *name)
  * Parses a host of the form <address>[:port]
  * :port is permitted if 'port' is not NULL
  */
-unsigned long ap_get_virthost_addr(const char *w, unsigned short *ports)
+unsigned long ap_get_virthost_addr(char *w, unsigned short *ports)
 {
     struct hostent *hep;
     unsigned long my_addr;
@@ -1539,7 +1568,7 @@ unsigned long ap_get_virthost_addr(const char *w, unsigned short *ports)
 	return htonl(INADDR_ANY);
     }
 
-    my_addr = ap_inet_addr(w);
+    my_addr = ap_inet_addr((char *)w);
     if (my_addr != INADDR_NONE) {
 	if (p != NULL)
 	    *p = ':';
@@ -1598,7 +1627,7 @@ char *ap_get_local_host(pool *a)
     str[MAXHOSTNAMELEN] = '\0';
     if ((!(p = gethostbyname(str))) || (!(server_hostname = find_fqdn(a, p)))) {
 	fprintf(stderr, "httpd: cannot determine local host name.\n");
-	fprintf(stderr, "Use ServerName to set it manually.\n");
+	fprintf(stderr, "Use the ServerName directive to set it manually.\n");
 	exit(1);
     }
 
@@ -1703,7 +1732,29 @@ API_EXPORT(char *) ap_uudecode(pool *p, const char *bufcoded)
     return bufplain;
 }
 
-#ifdef __EMX__
+static const char basis_64[] = 
+"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"; 
+ 
+API_EXPORT(char *) ap_uuencode(pool *a, char *string) 
+{ 
+    int i, len = strlen(string); 
+    char *p; 
+    char *encoded = (char *) ap_pcalloc(a, (len+2) / 3 * 4); 
+ 
+    p = encoded; 
+    for (i = 0; i < len; i += 3) { 
+        *p++ = basis_64[string[i] >> 2]; 
+        *p++ = basis_64[((string[i] & 0x3) << 4) | ((int) (string[i + 1] & 0xF0) >> 4)]; 
+        *p++ = basis_64[((string[i + 1] & 0xF) << 2) | ((int) (string[i + 2] & 0xC0) >> 6)]; 
+        *p++ = basis_64[string[i + 2] & 0x3F]; 
+    } 
+    *p-- = '\0'; 
+    *p-- = '='; 
+    *p-- = '='; 
+    return encoded; 
+} 
+
+#ifdef OS2
 void os2pathname(char *path)
 {
     char newpath[MAX_STRING_LEN];
@@ -1750,3 +1801,77 @@ double difftime(time_t time1, time_t time0)
     return (time1 - time0);
 }
 #endif
+
+/* we want to downcase the type/subtype for comparison purposes
+ * but nothing else because ;parameter=foo values are case sensitive.
+ * XXX: in truth we want to downcase parameter names... but really,
+ * apache has never handled parameters and such correctly.  You
+ * also need to compress spaces and such to be able to compare
+ * properly. -djg
+ */
+API_EXPORT(void) ap_content_type_tolower(char *str)
+{
+    char *semi;
+
+    semi = strchr(str, ';');
+    if (semi) {
+	*semi = '\0';
+    }
+    while (*str) {
+	*str = ap_tolower(*str);
+	++str;
+    }
+    if (semi) {
+	*semi = ';';
+    }
+}
+
+/*
+ * Given a string, replace any bare " with \" .
+ */
+API_EXPORT(char *) ap_escape_quotes (pool *p, const char *instring)
+{
+    int newlen = 0;
+    const char *inchr = instring;
+    char *outchr, *outstring;
+
+    /*
+     * Look through the input string, jogging the length of the output
+     * string up by an extra byte each time we find an unescaped ".
+     */
+    while (*inchr != '\0') {
+	newlen++;
+        if (*inchr == '"') {
+	    newlen++;
+	}
+	/*
+	 * If we find a slosh, and it's not the last byte in the string,
+	 * it's escaping something - advance past both bytes.
+	 */
+	if ((*inchr == '\\') && (inchr[1] != '\0')) {
+	    inchr++;
+	}
+	inchr++;
+    }
+    outstring = ap_palloc(p, newlen + 1);
+    inchr = instring;
+    outchr = outstring;
+    /*
+     * Now copy the input string to the output string, inserting a slosh
+     * in front of every " that doesn't already have one.
+     */
+    while (*inchr != '\0') {
+	if ((*inchr == '\\') && (inchr[1] != '\0')) {
+	    *outchr++ = *inchr++;
+	    *outchr++ = *inchr++;
+	}
+	if (*inchr == '"') {
+	    *outchr++ = '\\';
+	}
+	if (*inchr != '\0') {
+	    *outchr++ = *inchr++;
+	}
+    }
+    *outchr = '\0';
+    return outstring;
+}
