@@ -100,21 +100,26 @@ static char **create_argv(pool *p, char *path, char *user, char *group,
 
     /* count the number of keywords */
 
-    for (x = 0, numwords = 1; args[x]; x++)
-	if (args[x] == '+')
+    for (x = 0, numwords = 1; args[x]; x++) {
+        if (args[x] == '+') {
 	    ++numwords;
+	}
+    }
 
     if (numwords > APACHE_ARG_MAX - 5) {
 	numwords = APACHE_ARG_MAX - 5;	/* Truncate args to prevent overrun */
     }
     av = (char **) ap_palloc(p, (numwords + 5) * sizeof(char *));
 
-    if (path)
+    if (path) {
 	av[idx++] = path;
-    if (user)
+    }
+    if (user) {
 	av[idx++] = user;
-    if (group)
+    }
+    if (group) {
 	av[idx++] = group;
+    }
 
     av[idx++] = av0;
 
@@ -134,11 +139,11 @@ static char *http2env(pool *a, char *w)
     char *cp = res;
 
     while (*++cp) {
-	if (!isalnum(*cp) && *cp != '_') {
+	if (!ap_isalnum(*cp) && *cp != '_') {
 	    *cp = '_';
 	}
 	else {
-	    *cp = toupper(*cp);
+	    *cp = ap_toupper(*cp);
 	}
     }
 
@@ -147,7 +152,7 @@ static char *http2env(pool *a, char *w)
 
 API_EXPORT(char **) ap_create_environment(pool *p, table *t)
 {
-    array_header *env_arr = table_elts(t);
+    array_header *env_arr = ap_table_elts(t);
     table_entry *elts = (table_entry *) env_arr->elts;
     char **env = (char **) ap_palloc(p, (env_arr->nelts + 2) * sizeof(char *));
     int i, j;
@@ -157,19 +162,21 @@ API_EXPORT(char **) ap_create_environment(pool *p, table *t)
     j = 0;
     if (!ap_table_get(t, "TZ")) {
 	tz = getenv("TZ");
-	if (tz != NULL)
+	if (tz != NULL) {
 	    env[j++] = ap_pstrcat(p, "TZ=", tz, NULL);
+	}
     }
     for (i = 0; i < env_arr->nelts; ++i) {
-	if (!elts[i].key)
+        if (!elts[i].key) {
 	    continue;
+	}
 	env[j] = ap_pstrcat(p, elts[i].key, "=", elts[i].val, NULL);
 	whack = env[j];
-	if (isdigit(*whack)) {
+	if (ap_isdigit(*whack)) {
 	    *whack++ = '_';
 	}
 	while (*whack != '=') {
-	    if (!isalnum(*whack) && *whack != '_') {
+	    if (!ap_isalnum(*whack) && *whack != '_') {
 		*whack = '_';
 	    }
 	    ++whack;
@@ -183,7 +190,7 @@ API_EXPORT(char **) ap_create_environment(pool *p, table *t)
 
 API_EXPORT(void) ap_add_common_vars(request_rec *r)
 {
-    table *e = r->subprocess_env;
+    table *e;
     server_rec *s = r->server;
     conn_rec *c = r->connection;
     const char *rem_logname;
@@ -192,86 +199,109 @@ API_EXPORT(void) ap_add_common_vars(request_rec *r)
     char *env_temp;
 #endif
     const char *host;
-
-    array_header *hdrs_arr = table_elts(r->headers_in);
+    array_header *hdrs_arr = ap_table_elts(r->headers_in);
     table_entry *hdrs = (table_entry *) hdrs_arr->elts;
     int i;
 
+    /* use a temporary table which we'll overlap onto
+     * r->subprocess_env later
+     */
+    e = ap_make_table(r->pool, 25 + hdrs_arr->nelts);
+
     /* First, add environment vars from headers... this is as per
      * CGI specs, though other sorts of scripting interfaces see
      * the same vars...
      */
 
     for (i = 0; i < hdrs_arr->nelts; ++i) {
-	if (!hdrs[i].key)
+        if (!hdrs[i].key) {
 	    continue;
+	}
 
 	/* A few headers are special cased --- Authorization to prevent
 	 * rogue scripts from capturing passwords; content-type and -length
 	 * for no particular reason.
 	 */
 
-	if (!strcasecmp(hdrs[i].key, "Content-type"))
-	    ap_table_setn(e, "CONTENT_TYPE", hdrs[i].val);
-	else if (!strcasecmp(hdrs[i].key, "Content-length"))
-	    ap_table_setn(e, "CONTENT_LENGTH", hdrs[i].val);
+	if (!strcasecmp(hdrs[i].key, "Content-type")) {
+	    ap_table_addn(e, "CONTENT_TYPE", hdrs[i].val);
+	}
+	else if (!strcasecmp(hdrs[i].key, "Content-length")) {
+	    ap_table_addn(e, "CONTENT_LENGTH", hdrs[i].val);
+	}
 	/*
 	 * You really don't want to disable this check, since it leaves you
 	 * wide open to CGIs stealing passwords and people viewing them
 	 * in the environment with "ps -e".  But, if you must...
 	 */
 #ifndef SECURITY_HOLE_PASS_AUTHORIZATION
-	else if (!strcasecmp(hdrs[i].key, "Authorization") ||
-                 !strcasecmp(hdrs[i].key, "Proxy-Authorization"))
+	else if (!strcasecmp(hdrs[i].key, "Authorization") 
+		 || !strcasecmp(hdrs[i].key, "Proxy-Authorization")) {
 	    continue;
+	}
 #endif
-	else
-	    ap_table_setn(e, http2env(r->pool, hdrs[i].key), hdrs[i].val);
+	else {
+	    ap_table_addn(e, http2env(r->pool, hdrs[i].key), hdrs[i].val);
+	}
     }
 
-    if (!(env_path = getenv("PATH")))
+    if (!(env_path = getenv("PATH"))) {
 	env_path = DEFAULT_PATH;
+    }
 
 #ifdef WIN32
-    if (env_temp = getenv("SystemRoot"))
-        ap_table_setn(e, "SystemRoot", env_temp);         
-    if (env_temp = getenv("COMSPEC"))
-        ap_table_setn(e, "COMSPEC", env_temp);            
-    if (env_temp = getenv("WINDIR"))
-        ap_table_setn(e, "WINDIR", env_temp);             
+    if (env_temp = getenv("SystemRoot")) {
+        ap_table_addn(e, "SystemRoot", env_temp);         
+    }
+    if (env_temp = getenv("COMSPEC")) {
+        ap_table_addn(e, "COMSPEC", env_temp);            
+    }
+    if (env_temp = getenv("WINDIR")) {
+        ap_table_addn(e, "WINDIR", env_temp);
+    }
 #endif
 
-    ap_table_setn(e, "PATH", env_path);
-    ap_table_setn(e, "SERVER_SOFTWARE", ap_get_server_version());
-    ap_table_setn(e, "SERVER_NAME", ap_get_server_name(r));
-    ap_table_setn(e, "SERVER_PORT", ap_psprintf(r->pool, "%u", ap_get_server_port(r)));
+    ap_table_addn(e, "PATH", env_path);
+    ap_table_setn(e, "SERVER_SIGNATURE", ap_psignature("", r));
+    ap_table_addn(e, "SERVER_SOFTWARE", ap_get_server_version());
+    ap_table_addn(e, "SERVER_NAME", ap_get_server_name(r));
+    ap_table_addn(e, "SERVER_PORT",
+		  ap_psprintf(r->pool, "%u", ap_get_server_port(r)));
     host = ap_get_remote_host(c, r->per_dir_config, REMOTE_HOST);
     if (host) {
-	ap_table_setn(e, "REMOTE_HOST", host);
+	ap_table_addn(e, "REMOTE_HOST", host);
     }
-    ap_table_setn(e, "REMOTE_ADDR", c->remote_ip);
-    ap_table_setn(e, "DOCUMENT_ROOT", ap_document_root(r));	/* Apache */
-    ap_table_setn(e, "SERVER_ADMIN", s->server_admin);	/* Apache */
-    ap_table_setn(e, "SCRIPT_FILENAME", r->filename);	/* Apache */
+    ap_table_addn(e, "REMOTE_ADDR", c->remote_ip);
+    ap_table_addn(e, "DOCUMENT_ROOT", ap_document_root(r));	/* Apache */
+    ap_table_addn(e, "SERVER_ADMIN", s->server_admin);	/* Apache */
+    ap_table_addn(e, "SCRIPT_FILENAME", r->filename);	/* Apache */
 
-    ap_table_setn(e, "REMOTE_PORT", ap_psprintf(r->pool, "%d", ntohs(c->remote_addr.sin_port)));
+    ap_table_addn(e, "REMOTE_PORT",
+		  ap_psprintf(r->pool, "%d", ntohs(c->remote_addr.sin_port)));
 
-    if (c->user)
-	ap_table_setn(e, "REMOTE_USER", c->user);
-    if (c->ap_auth_type)
-	ap_table_setn(e, "AUTH_TYPE", c->ap_auth_type);
+    if (c->user) {
+	ap_table_addn(e, "REMOTE_USER", c->user);
+    }
+    if (c->ap_auth_type) {
+	ap_table_addn(e, "AUTH_TYPE", c->ap_auth_type);
+    }
     rem_logname = ap_get_remote_logname(r);
-    if (rem_logname)
-	ap_table_setn(e, "REMOTE_IDENT", ap_pstrdup(r->pool, rem_logname));
+    if (rem_logname) {
+	ap_table_addn(e, "REMOTE_IDENT", ap_pstrdup(r->pool, rem_logname));
+    }
 
     /* Apache custom error responses. If we have redirected set two new vars */
 
     if (r->prev) {
-	if (r->prev->args)
-	    ap_table_setn(e, "REDIRECT_QUERY_STRING", r->prev->args);
-	if (r->prev->uri)
-	    ap_table_setn(e, "REDIRECT_URL", r->prev->uri);
+        if (r->prev->args) {
+	    ap_table_addn(e, "REDIRECT_QUERY_STRING", r->prev->args);
+	}
+	if (r->prev->uri) {
+	    ap_table_addn(e, "REDIRECT_URL", r->prev->uri);
+	}
     }
+
+    ap_overlap_tables(r->subprocess_env, e, AP_OVERLAP_TABLES_SET);
 }
 
 /* This "cute" little function comes about because the path info on
@@ -286,12 +316,13 @@ API_EXPORT(int) ap_find_path_info(const char *uri, const char *path_info)
 
     while (lu-- && lp-- && uri[lu] == path_info[lp]);
 
-    if (lu == -1)
+    if (lu == -1) {
 	lu = 0;
+    }
 
-    while (uri[lu] != '\0' && uri[lu] != '/')
-	lu++;
-
+    while (uri[lu] != '\0' && uri[lu] != '/') {
+        lu++;
+    }
     return lu;
 }
 
@@ -302,19 +333,23 @@ static char *original_uri(request_rec *r)
 {
     char *first, *last;
 
-    if (r->the_request == NULL)
+    if (r->the_request == NULL) {
 	return (char *) ap_pcalloc(r->pool, 1);
+    }
 
     first = r->the_request;	/* use the request-line */
 
-    while (*first && !isspace(*first))
+    while (*first && !ap_isspace(*first)) {
 	++first;		/* skip over the method */
-    while (isspace(*first))
+    }
+    while (ap_isspace(*first)) {
 	++first;		/*   and the space(s)   */
+    }
 
     last = first;
-    while (*last && !isspace(*last))
+    while (*last && !ap_isspace(*last)) {
 	++last;			/* end at next whitespace */
+    }
 
     return ap_pstrndup(r->pool, first, last - first);
 }
@@ -337,8 +372,9 @@ API_EXPORT(void) ap_add_cgi_vars(request_rec *r)
 
     if (!strcmp(r->protocol, "INCLUDED")) {
 	ap_table_setn(e, "SCRIPT_NAME", r->uri);
-	if (r->path_info && *r->path_info)
+	if (r->path_info && *r->path_info) {
 	    ap_table_setn(e, "PATH_INFO", r->path_info);
+	}
     }
     else if (!r->path_info || !*r->path_info) {
 	ap_table_setn(e, "SCRIPT_NAME", r->uri);
@@ -346,8 +382,8 @@ API_EXPORT(void) ap_add_cgi_vars(request_rec *r)
     else {
 	int path_info_start = ap_find_path_info(r->uri, r->path_info);
 
-	ap_table_setn(e, "SCRIPT_NAME", ap_pstrndup(r->pool, r->uri,
-					     path_info_start));
+	ap_table_setn(e, "SCRIPT_NAME",
+		      ap_pstrndup(r->pool, r->uri, path_info_start));
 
 	ap_table_setn(e, "PATH_INFO", r->path_info);
     }
@@ -358,15 +394,16 @@ API_EXPORT(void) ap_add_cgi_vars(request_rec *r)
 	 * Need to re-escape it for this, since the entire URI was
 	 * un-escaped before we determined where the PATH_INFO began.
 	 */
-	request_rec *pa_req = ap_sub_req_lookup_uri(escape_uri(r->pool, r->path_info),
-						 r);
+	request_rec *pa_req;
+
+	pa_req = ap_sub_req_lookup_uri(escape_uri(r->pool, r->path_info), r);
 
 	if (pa_req->filename) {
 #ifdef WIN32
 	    char buffer[HUGE_STRING_LEN];
 #endif
 	    char *pt = ap_pstrcat(r->pool, pa_req->filename, pa_req->path_info,
-			       NULL);
+				  NULL);
 #ifdef WIN32
 	    /* We need to make this a real Windows path name */
 	    GetFullPathName(pt, HUGE_STRING_LEN, buffer, NULL);
@@ -380,37 +417,61 @@ API_EXPORT(void) ap_add_cgi_vars(request_rec *r)
 }
 
 
-static int scan_script_header_err_core(request_rec *r, char *buffer,
-		 int (*getsfunc) (char *, int, void *), void *getsfunc_data)
+static int set_cookie_doo_doo(void *v, const char *key, const char *val)
+{
+    ap_table_addn(v, key, val);
+    return 1;
+}
+
+API_EXPORT(int) ap_scan_script_header_err_core(request_rec *r, char *buffer,
+				       int (*getsfunc) (char *, int, void *),
+				       void *getsfunc_data)
 {
     char x[MAX_STRING_LEN];
     char *w, *l;
     int p;
     int cgi_status = HTTP_OK;
+    table *merge;
+    table *cookie_table;
 
-    if (buffer)
+    if (buffer) {
 	*buffer = '\0';
+    }
     w = buffer ? buffer : x;
 
     ap_hard_timeout("read script header", r);
 
+    /* temporary place to hold headers to merge in later */
+    merge = ap_make_table(r->pool, 10);
+
+    /* The HTTP specification says that it is legal to merge duplicate
+     * headers into one.  Some browsers that support Cookies don't like
+     * merged headers and prefer that each Set-Cookie header is sent
+     * separately.  Lets humour those browsers by not merging.
+     * Oh what a pain it is.
+     */
+    cookie_table = ap_make_table(r->pool, 2);
+    ap_table_do(set_cookie_doo_doo, cookie_table, r->err_headers_out, "Set-Cookie", NULL);
+
     while (1) {
 
 	if ((*getsfunc) (w, MAX_STRING_LEN - 1, getsfunc_data) == 0) {
 	    ap_kill_timeout(r);
-	    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
-			"Premature end of script headers: %s", r->filename);
-	    return SERVER_ERROR;
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
+			  "Premature end of script headers: %s", r->filename);
+	    return HTTP_INTERNAL_SERVER_ERROR;
 	}
 
 	/* Delete terminal (CR?)LF */
 
 	p = strlen(w);
 	if (p > 0 && w[p - 1] == '\n') {
-	    if (p > 1 && w[p - 2] == '\015')
+	    if (p > 1 && w[p - 2] == '\015') {
 		w[p - 2] = '\0';
-	    else
+	    }
+	    else {
 		w[p - 1] = '\0';
+	    }
 	}
 
 	/*
@@ -432,41 +493,76 @@ static int scan_script_header_err_core(request_rec *r, char *buffer,
 	    if ((cgi_status == HTTP_OK) && (r->method_number == M_GET)) {
 		cond_status = ap_meets_conditions(r);
 	    }
+	    ap_overlap_tables(r->err_headers_out, merge,
+		AP_OVERLAP_TABLES_MERGE);
+	    if (!ap_is_empty_table(cookie_table)) {
+		r->err_headers_out = ap_overlay_tables(r->pool,
+		    r->err_headers_out, cookie_table);
+	    }
 	    return cond_status;
 	}
 
 	/* if we see a bogus header don't ignore it. Shout and scream */
 
+#ifdef CHARSET_EBCDIC
+	    /* Chances are that we received an ASCII header text instead of
+	     * the expected EBCDIC header lines. Try to auto-detect:
+	     */
 	if (!(l = strchr(w, ':'))) {
-	    char malformed[(sizeof MALFORMED_MESSAGE) + 1 + MALFORMED_HEADER_LENGTH_TO_SHOW];
+	    int maybeASCII = 0, maybeEBCDIC = 0;
+	    char *cp;
+
+	    for (cp = w; *cp != '\0'; ++cp) {
+		if (isprint(*cp) && !isprint(os_toebcdic[*cp]))
+		    ++maybeEBCDIC;
+		if (!isprint(*cp) && isprint(os_toebcdic[*cp]))
+		    ++maybeASCII;
+		}
+	    if (maybeASCII > maybeEBCDIC) {
+		ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
+			 "CGI Interface Error: Script headers apparently ASCII: (CGI = %s)", r->filename);
+		ascii2ebcdic(w, w, cp - w);
+	    }
+	}
+#endif
+	if (!(l = strchr(w, ':'))) {
+	    char malformed[(sizeof MALFORMED_MESSAGE) + 1
+			   + MALFORMED_HEADER_LENGTH_TO_SHOW];
+
 	    strcpy(malformed, MALFORMED_MESSAGE);
 	    strncat(malformed, w, MALFORMED_HEADER_LENGTH_TO_SHOW);
 
-	    if (!buffer)
-		/* Soak up all the script output --- may save an outright kill */
-		while ((*getsfunc) (w, MAX_STRING_LEN - 1, getsfunc_data))
+	    if (!buffer) {
+		/* Soak up all the script output - may save an outright kill */
+	        while ((*getsfunc) (w, MAX_STRING_LEN - 1, getsfunc_data)) {
 		    continue;
+		}
+	    }
 
 	    ap_kill_timeout(r);
-	    ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
-			"%s: %s", malformed, r->filename);
-	    return SERVER_ERROR;
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
+			  "%s: %s", malformed, r->filename);
+	    return HTTP_INTERNAL_SERVER_ERROR;
 	}
 
 	*l++ = '\0';
-	while (*l && isspace(*l))
+	while (*l && ap_isspace(*l)) {
 	    ++l;
+	}
 
 	if (!strcasecmp(w, "Content-type")) {
+	    char *tmp;
 
 	    /* Nuke trailing whitespace */
 
 	    char *endp = l + strlen(l) - 1;
-	    while (endp > l && isspace(*endp))
+	    while (endp > l && ap_isspace(*endp)) {
 		*endp-- = '\0';
+	    }
 
-	    r->content_type = ap_pstrdup(r->pool, l);
-	    ap_str_tolower(r->content_type);
+	    tmp = ap_pstrdup(r->pool, l);
+	    ap_content_type_tolower(tmp);
+	    r->content_type = tmp;
 	}
 	/*
 	 * If the script returned a specific status, that's what
@@ -495,16 +591,11 @@ static int scan_script_header_err_core(request_rec *r, char *buffer,
 	    ap_update_mtime(r, mtime);
 	    ap_set_last_modified(r);
 	}
-	/* The HTTP specification says that it is legal to merge duplicate
-	 * headers into one.  Some browsers that support Cookies don't like
-	 * merged headers and prefer that each Set-Cookie header is sent
-	 * separately.  Lets humour those browsers.
-	 */
 	else if (!strcasecmp(w, "Set-Cookie")) {
-	    ap_table_add(r->err_headers_out, w, l);
+	    ap_table_add(cookie_table, w, l);
 	}
 	else {
-	    ap_table_merge(r->err_headers_out, w, l);
+	    ap_table_add(merge, w, l);
 	}
     }
 }
@@ -514,9 +605,10 @@ static int getsfunc_FILE(char *buf, int len, void *f)
     return fgets(buf, len, (FILE *) f) != NULL;
 }
 
-API_EXPORT(int) ap_scan_script_header_err(request_rec *r, FILE *f, char *buffer)
+API_EXPORT(int) ap_scan_script_header_err(request_rec *r, FILE *f,
+					  char *buffer)
 {
-    return scan_script_header_err_core(r, buffer, getsfunc_FILE, f);
+    return ap_scan_script_header_err_core(r, buffer, getsfunc_FILE, f);
 }
 
 static int getsfunc_BUFF(char *w, int len, void *fb)
@@ -525,39 +617,47 @@ static int getsfunc_BUFF(char *w, int len, void *fb)
 }
 
 API_EXPORT(int) ap_scan_script_header_err_buff(request_rec *r, BUFF *fb,
-					    char *buffer)
+					       char *buffer)
 {
-    return scan_script_header_err_core(r, buffer, getsfunc_BUFF, fb);
+    return ap_scan_script_header_err_core(r, buffer, getsfunc_BUFF, fb);
 }
 
 
 API_EXPORT(void) ap_send_size(size_t size, request_rec *r)
 {
     /* XXX: this -1 thing is a gross hack */
-    if (size == (size_t)-1)
+    if (size == (size_t)-1) {
 	ap_rputs("    -", r);
-    else if (!size)
+    }
+    else if (!size) {
 	ap_rputs("   0k", r);
-    else if (size < 1024)
+    }
+    else if (size < 1024) {
 	ap_rputs("   1k", r);
-    else if (size < 1048576)
+    }
+    else if (size < 1048576) {
 	ap_rprintf(r, "%4dk", (size + 512) / 1024);
-    else if (size < 103809024)
+    }
+    else if (size < 103809024) {
 	ap_rprintf(r, "%4.1fM", size / 1048576.0);
-    else
+    }
+    else {
 	ap_rprintf(r, "%4dM", (size + 524288) / 1048576);
+    }
 }
 
-#if defined(__EMX__) || defined(WIN32)
+#if defined(OS2) || defined(WIN32)
 static char **create_argv_cmd(pool *p, char *av0, const char *args, char *path)
 {
     register int x, n;
     char **av;
     char *w;
 
-    for (x = 0, n = 2; args[x]; x++)
-	if (args[x] == '+')
+    for (x = 0, n = 2; args[x]; x++) {
+        if (args[x] == '+') {
 	    ++n;
+	}
+    }
 
     /* Add extra strings to array. */
     n = n + 2;
@@ -580,14 +680,16 @@ static char **create_argv_cmd(pool *p, char *av0, const char *args, char *path)
 #endif
 
 
-API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, char **env, int shellcmd)
+API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0,
+			     char **env, int shellcmd)
 {
     int pid = 0;
 #if defined(RLIMIT_CPU)  || defined(RLIMIT_NPROC) || \
     defined(RLIMIT_DATA) || defined(RLIMIT_VMEM) || defined (RLIMIT_AS)
 
-    core_dir_config *conf =
-    (core_dir_config *) ap_get_module_config(r->per_dir_config, &core_module);
+    core_dir_config *conf;
+    conf = (core_dir_config *) ap_get_module_config(r->per_dir_config,
+						    &core_module);
 
 #endif
 
@@ -601,44 +703,58 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, cha
 #endif
 
 #ifdef RLIMIT_CPU
-    if (conf->limit_cpu != NULL)
-	if ((setrlimit(RLIMIT_CPU, conf->limit_cpu)) != 0)
+    if (conf->limit_cpu != NULL) {
+        if ((setrlimit(RLIMIT_CPU, conf->limit_cpu)) != 0) {
 	    ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-			"setrlimit: failed to set CPU usage limit");
+			 "setrlimit: failed to set CPU usage limit");
+	}
+    }
 #endif
 #ifdef RLIMIT_NPROC
-    if (conf->limit_nproc != NULL)
-	if ((setrlimit(RLIMIT_NPROC, conf->limit_nproc)) != 0)
+    if (conf->limit_nproc != NULL) {
+        if ((setrlimit(RLIMIT_NPROC, conf->limit_nproc)) != 0) {
 	    ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-			"setrlimit: failed to set process limit");
+			 "setrlimit: failed to set process limit");
+	}
+    }
 #endif
 #if defined(RLIMIT_AS)
-    if (conf->limit_mem != NULL)
-	if ((setrlimit(RLIMIT_AS, conf->limit_mem)) != 0)
+    if (conf->limit_mem != NULL) {
+        if ((setrlimit(RLIMIT_AS, conf->limit_mem)) != 0) {
 	    ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-			"setrlimit(RLIMIT_AS): failed to set memory usage limit");
+			 "setrlimit(RLIMIT_AS): failed to set memory "
+			 "usage limit");
+	}
+    }
 #elif defined(RLIMIT_DATA)
-    if (conf->limit_mem != NULL)
-	if ((setrlimit(RLIMIT_DATA, conf->limit_mem)) != 0)
+    if (conf->limit_mem != NULL) {
+        if ((setrlimit(RLIMIT_DATA, conf->limit_mem)) != 0) {
 	    ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-			"setrlimit(RLIMIT_DATA): failed to set memory usage limit");
+			 "setrlimit(RLIMIT_DATA): failed to set memory "
+			 "usage limit");
+	}
+    }
 #elif defined(RLIMIT_VMEM)
-    if (conf->limit_mem != NULL)
-	if ((setrlimit(RLIMIT_VMEM, conf->limit_mem)) != 0)
+    if (conf->limit_mem != NULL) {
+        if ((setrlimit(RLIMIT_VMEM, conf->limit_mem)) != 0) {
 	    ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-			"setrlimit(RLIMIT_VMEM): failed to set memory usage limit");
+			 "setrlimit(RLIMIT_VMEM): failed to set memory "
+			 "usage limit");
+	}
+    }
 #endif
 
-#ifdef __EMX__
+#ifdef OS2
     {
 	/* Additions by Alec Kloss, to allow exec'ing of scripts under OS/2 */
 	int is_script;
-	char interpreter[2048];	/* hope this is large enough for the interpreter path */
+	char interpreter[2048];	/* hope it's enough for the interpreter path */
 	FILE *program;
+
 	program = fopen(r->filename, "rt");
 	if (!program) {
-	    ap_log_error(APLOG_MARK, APLOG_ERR, r->server, "fopen(%s) failed",
-			r->filename);
+	    ap_log_rerror(APLOG_MARK, APLOG_ERR, r, "fopen(%s) failed",
+			 r->filename);
 	    return (pid);
 	}
 	fgets(interpreter, sizeof(interpreter), program);
@@ -656,13 +772,14 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, cha
 	    char *emxtemp;
 
 	    /* For OS/2 place the variables in the current
-	     * enviornment then it will be inherited. This way
+	     * environment then it will be inherited. This way
 	     * the program will also get all of OS/2's other SETs.
 	     */
-	    for (emxloop = 0; ((emxtemp = env[emxloop]) != NULL); emxloop++)
+	    for (emxloop = 0; ((emxtemp = env[emxloop]) != NULL); emxloop++) {
 		putenv(emxtemp);
+	    }
 
-	    /* Additions by Alec Kloss, to allow exec'ing of scripts under OS/2 */
+	    /* More additions by Alec Kloss for OS/2 */
 	    if (is_script) {
 		/* here's the stuff to run the interpreter */
 		execl(interpreter + 2, interpreter + 2, r->filename, NULL);
@@ -684,30 +801,34 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, cha
 	     * environment so that they will be inherited. This way
 	     * the program will also get all of OS/2's other SETs.
 	     */
-	    for (emxloop = 0; ((emxtemp = env[emxloop]) != NULL); emxloop++)
+	    for (emxloop = 0; ((emxtemp = env[emxloop]) != NULL); emxloop++) {
 		putenv(emxtemp);
+	    }
 
 	    if (strstr(strupr(r->filename), ".CMD") > 0) {
 		/* Special case to allow use of REXX commands as scripts. */
 		os2pathname(r->filename);
-		execv(SHELL_PATH, create_argv_cmd(r->pool, argv0, r->args, r->filename));
+		execv(SHELL_PATH, create_argv_cmd(r->pool, argv0, r->args,
+						  r->filename));
 	    }
-	    else
+	    else {
 		execv(r->filename,
-		    create_argv(r->pool, NULL, NULL, NULL, argv0, r->args));
+		      create_argv(r->pool, NULL, NULL, NULL, argv0, r->args));
+	    }
 	}
 	return (pid);
     }
 #elif defined(WIN32)
     {
-	/* Adapted from work by Alec Kloss, to allow exec'ing of scripts under OS/2 */
+	/* Adapted from Alec Kloss' work for OS/2 */
 	int is_script = 0;
 	int is_binary = 0;
-	char interpreter[2048];	/* hope this is large enough for the interpreter path */
+	char interpreter[2048];	/* hope it's enough for the interpreter path */
 	FILE *program;
 	int i, sz;
 	char *dot;
 	char *exename;
+        char *quoted_filename;
 	int is_exe = 0;
 	STARTUPINFO si;
 	PROCESS_INFORMATION pi;
@@ -719,78 +840,102 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, cha
 	memset(&pi, 0, sizeof(pi));
 
 	interpreter[0] = 0;
+	pid = -1;
 
-	exename = strrchr(r->filename, '/');
-	if (!exename)
-	    exename = strrchr(r->filename, '\\');
-	if (!exename)
-	    exename = r->filename;
-	else
-	    exename++;
-	dot = strrchr(exename, '.');
-	if (dot) {
-	    if (!strcasecmp(dot, ".BAT") ||
-		!strcasecmp(dot, ".CMD") ||
-		!strcasecmp(dot, ".EXE") ||
-		!strcasecmp(dot, ".COM"))
-		is_exe = 1;
-	}
+        quoted_filename = ap_pstrcat(r->pool, "\"", r->filename, "\"", NULL);
 
-	if (!is_exe) {
-	    program = fopen(r->filename, "rb");
-	    if (!program) {
-		ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-			    "fopen(%s) failed", r->filename);
-		return (pid);
-	    }
-	    sz = fread(interpreter, 1, sizeof(interpreter) - 1, program);
-	    if (sz < 0) {
-		ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-			    "fread of %s failed", r->filename);
-		fclose(program);
-		return (pid);
-	    }
-	    interpreter[sz] = 0;
-	    fclose(program);
-	    if (!strncmp(interpreter, "#!", 2)) {
-		is_script = 1;
-		for (i = 2; i < sizeof(interpreter); i++) {
-		    if ((interpreter[i] == '\r') ||
-			(interpreter[i] == '\n'))
-			break;
-		}
-		interpreter[i] = 0;
-		for (i = 2; interpreter[i] == ' '; ++i)
-		    ;
-		memmove(interpreter+2,interpreter+i,strlen(interpreter+i)+1);
-	    }
-	    else {
-                        /* Check to see if it's a executable */
-                IMAGE_DOS_HEADER *hdr = (IMAGE_DOS_HEADER*)interpreter;
-                if (hdr->e_magic == IMAGE_DOS_SIGNATURE && hdr->e_cblp < 512)
-                    is_binary = 1;
-	    }
-	}
+        if (!shellcmd) {
+            exename = strrchr(r->filename, '/');
+            if (!exename) {
+                exename = strrchr(r->filename, '\\');
+            }
+            if (!exename) {
+                exename = r->filename;
+            }
+            else {
+                exename++;
+            }
+            dot = strrchr(exename, '.');
+            if (dot) {
+                if (!strcasecmp(dot, ".BAT")
+                    || !strcasecmp(dot, ".CMD")
+                    || !strcasecmp(dot, ".EXE")
+                    ||  !strcasecmp(dot, ".COM")) {
+                    is_exe = 1;
+                }
+            }
 
-	/*
-	 * Make child process use hPipeOutputWrite as standard out,
-	 * and make sure it does not show on screen.
-	 */
-	si.cb = sizeof(si);
-	si.dwFlags     = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
-	si.wShowWindow = SW_HIDE;
-	si.hStdInput   = pinfo->hPipeInputRead;
-	si.hStdOutput  = pinfo->hPipeOutputWrite;
-	si.hStdError   = pinfo->hPipeErrorWrite;
-	
-	pid = -1;      
-	if ((!r->args) || (!r->args[0]) || strchr(r->args, '=')) { 
+            if (!is_exe) {
+                program = fopen(r->filename, "rb");
+                if (!program) {
+                    ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
+                                 "fopen(%s) failed", r->filename);
+                    return (pid);
+                }
+                sz = fread(interpreter, 1, sizeof(interpreter) - 1, program);
+                if (sz < 0) {
+                    ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
+                                 "fread of %s failed", r->filename);
+                    fclose(program);
+                    return (pid);
+                }
+                interpreter[sz] = 0;
+                fclose(program);
+                if (!strncmp(interpreter, "#!", 2)) {
+                    is_script = 1;
+                    for (i = 2; i < sizeof(interpreter); i++) {
+                        if ((interpreter[i] == '\r')
+                            || (interpreter[i] == '\n')) {
+                            break;
+                        }
+                    }
+                    interpreter[i] = 0;
+                    for (i = 2; interpreter[i] == ' '; ++i)
+                        ;
+                    memmove(interpreter+2,interpreter+i,strlen(interpreter+i)+1);
+                }
+                else {
+                    /* Check to see if it's a executable */
+                    IMAGE_DOS_HEADER *hdr = (IMAGE_DOS_HEADER*)interpreter;
+                    if (hdr->e_magic == IMAGE_DOS_SIGNATURE && hdr->e_cblp < 512) {
+                        is_binary = 1;
+                    }
+                }
+            }
+            /* Bail out if we haven't figured out what kind of
+             * file this is by now..
+             */
+            if (!is_exe && !is_script && !is_binary) {
+                ap_log_rerror(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, r,
+                             "%s is not executable; ensure interpreted scripts have "
+                             "\"#!\" first line", 
+                             r->filename);
+                return (pid);
+            }
+        }
+
+        if (shellcmd) {
+            char *shell_cmd = "CMD.EXE /C ";
+            OSVERSIONINFO osver;
+            osver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+         
+            /*
+             * Use CMD.EXE for NT, COMMAND.COM for WIN95
+             */
+            if (GetVersionEx(&osver)) {
+                if (osver.dwPlatformId != VER_PLATFORM_WIN32_NT) {
+                    shell_cmd = "COMMAND.COM /C ";
+                }
+            }       
+            pCommand = ap_pstrcat(r->pool, shell_cmd, argv0, NULL);
+        }
+ 	else if ((!r->args) || (!r->args[0]) || strchr(r->args, '=')) { 
 	    if (is_exe || is_binary) {
 	        /*
 	         * When the CGI is a straight binary executable, 
 		 * we can run it as is
 	         */
-	        pCommand = r->filename;
+	        pCommand = quoted_filename;
 	    }
 	    else if (is_script) {
                 /* When an interpreter is needed, we need to create 
@@ -798,14 +943,14 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, cha
                  * followed by the CGI script name.  
 		 */
 	        pCommand = ap_pstrcat(r->pool, interpreter + 2, " ", 
-				      r->filename, NULL);
+				      quoted_filename, NULL);
 	    }
 	    else {
 	        /* If not an executable or script, just execute it
                  * from a command prompt.  
                  */
 	        pCommand = ap_pstrcat(r->pool, SHELL_PATH, " /C ", 
-				      r->filename, NULL);
+				      quoted_filename, NULL);
 	    }
 	}
 	else {
@@ -851,25 +996,35 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, cha
 	     * so now build the command line.
 	     */
 	    if (is_exe || is_binary) {
-	        pCommand = ap_pstrcat(r->pool, r->filename, " ", 
+	        pCommand = ap_pstrcat(r->pool, quoted_filename, " ", 
 				      arguments, NULL);
 	    }
 	    else if (is_script) {
 	        pCommand = ap_pstrcat(r->pool, interpreter + 2, " ", 
-				      r->filename, " ", arguments, NULL);
+				      quoted_filename, " ", arguments, NULL);
 	    }
 	    else {
 	        pCommand = ap_pstrcat(r->pool, SHELL_PATH, " /C ", 
-				      r->filename, " ", arguments, NULL);
+				      quoted_filename, " ", arguments, NULL);
 	    }
 	}
+
+	/*
+	 * Make child process use hPipeOutputWrite as standard out,
+	 * and make sure it does not show on screen.
+	 */
+	si.cb = sizeof(si);
+	si.dwFlags     = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
+	si.wShowWindow = SW_HIDE;
+	si.hStdInput   = pinfo->hPipeInputRead;
+	si.hStdOutput  = pinfo->hPipeOutputWrite;
+	si.hStdError   = pinfo->hPipeErrorWrite;
   
         /*
          * Win32's CreateProcess call requires that the environment
          * be passed in an environment block, a null terminated block of
          * null terminated strings.
-         */
-  
+         */  
         i = 0;
         iEnvBlockLen = 1;
         while (env[i]) {
@@ -933,10 +1088,10 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, cha
 	return (pid);
     }
 #else
-    if (ap_suexec_enabled &&
-	((r->server->server_uid != ap_user_id) ||
-	 (r->server->server_gid != ap_group_id) ||
-	 (!strncmp("/~", r->uri, 2)))) {
+    if (ap_suexec_enabled
+	&& ((r->server->server_uid != ap_user_id)
+	    || (r->server->server_gid != ap_group_id)
+	    || (!strncmp("/~", r->uri, 2)))) {
 
 	char *execuser, *grpname;
 	struct passwd *pw;
@@ -947,49 +1102,57 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, cha
 	    char *username = ap_pstrdup(r->pool, r->uri + 2);
 	    char *pos = strchr(username, '/');
 
-	    if (pos)
+	    if (pos) {
 		*pos = '\0';
+	    }
 
 	    if ((pw = getpwnam(username)) == NULL) {
-		ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-			    "getpwnam: invalid username %s", username);
+		ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
+			     "getpwnam: invalid username %s", username);
 		return (pid);
 	    }
 	    execuser = ap_pstrcat(r->pool, "~", pw->pw_name, NULL);
 	    user_gid = pw->pw_gid;
 
 	    if ((gr = getgrgid(user_gid)) == NULL) {
-		if ((grpname = ap_palloc(r->pool, 16)) == NULL)
+	        if ((grpname = ap_palloc(r->pool, 16)) == NULL) {
 		    return (pid);
-		else
+		}
+		else {
 		    ap_snprintf(grpname, 16, "%ld", (long) user_gid);
+		}
 	    }
-	    else
+	    else {
 		grpname = gr->gr_name;
+	    }
 	}
 	else {
 	    if ((pw = getpwuid(r->server->server_uid)) == NULL) {
-		ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-		            "getpwuid: invalid userid %ld",
-		            (long) r->server->server_uid);
+		ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
+			     "getpwuid: invalid userid %ld",
+			     (long) r->server->server_uid);
 		return (pid);
 	    }
 	    execuser = ap_pstrdup(r->pool, pw->pw_name);
 
 	    if ((gr = getgrgid(r->server->server_gid)) == NULL) {
-		ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
-		            "getgrgid: invalid groupid %ld",
-		            (long) r->server->server_gid);
+		ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
+			     "getgrgid: invalid groupid %ld",
+			     (long) r->server->server_gid);
 		return (pid);
 	    }
 	    grpname = gr->gr_name;
 	}
 
-	if (shellcmd)
-	    execle(SUEXEC_BIN, SUEXEC_BIN, execuser, grpname, argv0, NULL, env);
+	if (shellcmd) {
+	    execle(SUEXEC_BIN, SUEXEC_BIN, execuser, grpname, argv0,
+		   NULL, env);
+	}
 
-	else if ((!r->args) || (!r->args[0]) || strchr(r->args, '='))
-	    execle(SUEXEC_BIN, SUEXEC_BIN, execuser, grpname, argv0, NULL, env);
+	else if ((!r->args) || (!r->args[0]) || strchr(r->args, '=')) {
+	    execle(SUEXEC_BIN, SUEXEC_BIN, execuser, grpname, argv0,
+		   NULL, env);
+	}
 
 	else {
 	    execve(SUEXEC_BIN,
@@ -999,16 +1162,19 @@ API_EXPORT(int) ap_call_exec(request_rec *r, child_info *pinfo, char *argv0, cha
 	}
     }
     else {
-	if (shellcmd)
+        if (shellcmd) {
 	    execle(SHELL_PATH, SHELL_PATH, "-c", argv0, NULL, env);
+	}
 
-	else if ((!r->args) || (!r->args[0]) || strchr(r->args, '='))
+	else if ((!r->args) || (!r->args[0]) || strchr(r->args, '=')) {
 	    execle(r->filename, argv0, NULL, env);
+	}
 
-	else
+	else {
 	    execve(r->filename,
 		   create_argv(r->pool, NULL, NULL, NULL, argv0, r->args),
 		   env);
+	}
     }
     return (pid);
 #endif
