@@ -97,7 +97,21 @@ DEF_Explain
  * FIXME: no check for r->assbackwards, whatever that is.
  */
 
-int ap_proxy_connect_handler(request_rec *r, struct cache_req *c, char *url,
+static int
+allowed_port(proxy_server_conf *conf, int port)
+{
+    int i;
+    int *list = (int *) conf->allowed_connect_ports->elts;
+
+    for(i = 0; i < conf->allowed_connect_ports->nelts; i++) {
+	if(port == list[i])
+	    return 1;
+    }
+    return 0;
+}
+
+
+int ap_proxy_connect_handler(request_rec *r, cache_req *c, char *url,
 			  const char *proxyhost, int proxyport)
 {
     struct sockaddr_in server;
@@ -137,13 +151,18 @@ int ap_proxy_connect_handler(request_rec *r, struct cache_req *c, char *url,
 	    return ap_proxyerror(r, "Connect to remote machine blocked");
     }
 
-    switch (port) {
-	case DEFAULT_HTTPS_PORT:
+    /* Check if it is an allowed port */
+    if (conf->allowed_connect_ports->nelts == 0) {
+	/* Default setting if not overridden by AllowCONNECT */
+	switch (port) {
+	    case DEFAULT_HTTPS_PORT:
 	    case DEFAULT_SNEWS_PORT:
-	    break;
-	default:
-	    return HTTP_SERVICE_UNAVAILABLE;
-    }
+		break;
+	    default:
+		return HTTP_FORBIDDEN;
+	}
+    } else if(!allowed_port(conf, port))
+	return HTTP_FORBIDDEN;
 
     if (proxyhost) {
 	Explain2("CONNECT to remote proxy %s on port %d", proxyhost, proxyport);
@@ -160,9 +179,9 @@ int ap_proxy_connect_handler(request_rec *r, struct cache_req *c, char *url,
 
     sock = ap_psocket(r->pool, PF_INET, SOCK_STREAM, IPPROTO_TCP);
     if (sock == -1) {
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 		    "proxy: error creating socket");
-	return SERVER_ERROR;
+	return HTTP_INTERNAL_SERVER_ERROR;
     }
 
 #ifndef WIN32
@@ -173,7 +192,7 @@ int ap_proxy_connect_handler(request_rec *r, struct cache_req *c, char *url,
 	    "found, you probably need to rebuild Apache with a "
 	    "larger FD_SETSIZE", sock, FD_SETSIZE);
 	ap_pclosesocket(r->pool, sock);
-	return SERVER_ERROR;
+	return HTTP_INTERNAL_SERVER_ERROR;
     }
 #endif
 
