@@ -62,6 +62,7 @@
 #include "multithread.h"
 #include "http_log.h"
 #include "util_uri.h"
+#include "util_date.h"	/* get ap_checkmask() decl. */
 
 static int proxy_match_ipaddr(struct dirconn_entry *This, request_rec *r);
 static int proxy_match_domainname(struct dirconn_entry *This, request_rec *r);
@@ -75,18 +76,18 @@ int ap_proxy_hex2c(const char *x)
 
 #ifndef CHARSET_EBCDIC
     ch = x[0];
-    if (isdigit(ch))
+    if (ap_isdigit(ch))
 	i = ch - '0';
-    else if (isupper(ch))
+    else if (ap_isupper(ch))
 	i = ch - ('A' - 10);
     else
 	i = ch - ('a' - 10);
     i <<= 4;
 
     ch = x[1];
-    if (isdigit(ch))
+    if (ap_isdigit(ch))
 	i += ch - '0';
-    else if (isupper(ch))
+    else if (ap_isupper(ch))
 	i += ch - ('A' - 10);
     else
 	i += ch - ('a' - 10);
@@ -188,7 +189,7 @@ char *
 	    }
 	}
 /* recode it, if necessary */
-	if (!isalnum(ch) && !strchr(allowed, ch)) {
+	if (!ap_isalnum(ch) && !strchr(allowed, ch)) {
 	    ap_proxy_c2hex(ch, &y[j]);
 	    j += 2;
 	}
@@ -259,7 +260,7 @@ char *
 	*(strp++) = '\0';
 
 	for (i = 0; strp[i] != '\0'; i++)
-	    if (!isdigit(strp[i]))
+	    if (!ap_isdigit(strp[i]))
 		break;
 
 	if (i == 0 || strp[i] != '\0')
@@ -273,7 +274,7 @@ char *
 	return "Missing host in URL";
 /* check hostname syntax */
     for (i = 0; host[i] != '\0'; i++)
-	if (!isdigit(host[i]) && host[i] != '.')
+	if (!ap_isdigit(host[i]) && host[i] != '.')
 	    break;
     /* must be an IP address */
 #ifdef WIN32
@@ -304,8 +305,8 @@ static const char * const lwday[7] =
  * sscanf and sprintf. However, if the date is already correctly
  * formatted, then it exits very quickly.
  */
-char *
-     ap_proxy_date_canon(pool *p, char *x)
+const char *
+     ap_proxy_date_canon(pool *p, const char *x)
 {
     int wk, mday, year, hour, min, sec, mon;
     char *q, month[4], zone[4], week[4];
@@ -353,107 +354,156 @@ char *
     if (mon == 12)
 	return x;
 
-    if (strlen(x)+1 < 30)
-	x = ap_palloc(p, 30);
-    ap_snprintf(x, 30, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT", ap_day_snames[wk], mday,
+    q = ap_palloc(p, 30);
+    ap_snprintf(q, 30, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT", ap_day_snames[wk], mday,
 		ap_month_snames[mon], year, hour, min, sec);
-    return x;
+    return q;
 }
 
+
+/* NOTE: This routine is taken from http_protocol::getline()
+ * because the old code found in the proxy module was too
+ * difficult to understand and maintain.
+ */
+/* Get a line of protocol input, including any continuation lines
+ * caused by MIME folding (or broken clients) if fold != 0, and place it
+ * in the buffer s, of size n bytes, without the ending newline.
+ *
+ * Returns -1 on error, or the length of s.
+ *
+ * Note: Because bgets uses 1 char for newline and 1 char for NUL,
+ *       the most we can get is (n - 2) actual characters if it
+ *       was ended by a newline, or (n - 1) characters if the line
+ *       length exceeded (n - 1).  So, if the result == (n - 1),
+ *       then the actual input line exceeded the buffer length,
+ *       and it would be a good idea for the caller to puke 400 or 414.
+ */
+static int proxy_getline(char *s, int n, BUFF *in, int fold)
+{
+    char *pos, next;
+    int retval;
+    int total = 0;
+
+    pos = s;
+
+    do {
+        retval = ap_bgets(pos, n, in);     /* retval == -1 if error, 0 if EOF */
+
+        if (retval <= 0)
+            return ((retval < 0) && (total == 0)) ? -1 : total;
+
+        /* retval is the number of characters read, not including NUL      */
+
+        n -= retval;            /* Keep track of how much of s is full     */
+        pos += (retval - 1);    /* and where s ends                        */
+        total += retval;        /* and how long s has become               */
+
+        if (*pos == '\n') {     /* Did we get a full line of input?        */
+            *pos = '\0';
+            --total;
+            ++n;
+        }
+        else
+            return total;       /* if not, input line exceeded buffer size */
+
+        /* Continue appending if line folding is desired and
+         * the last line was not empty and we have room in the buffer and
+         * the next line begins with a continuation character.
+         */
+    } while (fold && (retval != 1) && (n > 1)
+                  && (ap_blookc(&next, in) == 1)
+                  && ((next == ' ') || (next == '\t')));
+
+    return total;
+}
+
+
 /*
  * Reads headers from a buffer and returns an array of headers.
  * Returns NULL on file error
+ * This routine tries to deal with too long lines and continuation lines.
+ * @@@: XXX: FIXME: currently the headers are passed thru un-merged. 
+ * Is that okay, or should they be collapsed where possible?
  */
-array_header *
-             ap_proxy_read_headers(pool *p, char *buffer, int size, BUFF *f)
+table *ap_proxy_read_headers(request_rec *r, char *buffer, int size, BUFF *f)
 {
-    int gotcr, len, i, j;
-    array_header *resp_hdrs;
-    struct hdr_entry *hdr;
-    char *strp;
+    table *resp_hdrs;
+    int len;
+    char *value, *end;
+    char field[MAX_STRING_LEN];
 
-    resp_hdrs = ap_make_array(p, 10, sizeof(struct hdr_entry));
-    hdr = NULL;
+    resp_hdrs = ap_make_table(r->pool, 20);
 
-    gotcr = 1;
-    for (;;) {
-	len = ap_bgets(buffer, size, f);
-	if (len == -1)
-	    return NULL;
-	if (len == 0)
-	    break;
-	if (buffer[len - 1] == '\n') {
-	    buffer[--len] = '\0';
-	    i = 1;
-	}
-	else
-	    i = 0;
+    /*
+     * Read header lines until we get the empty separator line, a read error,
+     * the connection closes (EOF), or we timeout.
+     */
+    while ((len = proxy_getline(buffer, size, f, 1)) > 0) {
+	
+	if (!(value = strchr(buffer, ':'))) {     /* Find the colon separator */
 
-	if (!gotcr || buffer[0] == ' ' || buffer[0] == '\t') {
-	    /* a continuation header */
-	    if (hdr == NULL) {
-		/* error!! */
-		if (!i) {
-		    i = ap_bskiplf(f);
-		    if (i == -1)
-			return NULL;
-		}
-		gotcr = 1;
-		continue;
+	    /* Buggy MS IIS servers sometimes return invalid headers
+	     * (an extra "HTTP/1.0 200, OK" line sprinkled in between
+	     * the usual MIME headers). Try to deal with it in a sensible
+	     * way, but log the fact.
+	     * XXX: The mask check is buggy if we ever see an HTTP/1.10 */
+
+	    if (!ap_checkmask(buffer, "HTTP/#.# ###*")) {
+		/* Nope, it wasn't even an extra HTTP header. Give up. */
+		return NULL;
 	    }
-	    hdr->value = ap_pstrcat(p, hdr->value, buffer, NULL);
+
+	    ap_log_error(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, r->server,
+			 "proxy: Ignoring duplicate HTTP header "
+			 "returned by %s (%s)", r->uri, r->method);
+	    continue;
 	}
-	else if (gotcr && len == 0)
-	    break;
-	else {
-	    strp = strchr(buffer, ':');
-	    if (strp == NULL) {
-		/* error!! */
-		if (!gotcr) {
-		    i = ap_bskiplf(f);
-		    if (i == -1)
-			return NULL;
-		}
-		gotcr = 1;
-		hdr = NULL;
-		continue;
+
+        *value = '\0';
+        ++value;
+	/* XXX: RFC2068 defines only SP and HT as whitespace, this test is
+	 * wrong... and so are many others probably.
+	 */
+        while (ap_isspace(*value))
+            ++value;            /* Skip to start of value   */
+
+	/* should strip trailing whitespace as well */
+	for (end = &value[strlen(value)-1]; end > value && ap_isspace(*end); --end)
+	    *end = '\0';
+
+        ap_table_add(resp_hdrs, buffer, value);
+
+	/* the header was too long; at the least we should skip extra data */
+	if (len >= size - 1) { 
+	    while ((len = proxy_getline(field, MAX_STRING_LEN, f, 1))
+		    >= MAX_STRING_LEN - 1) {
+		/* soak up the extra data */
 	    }
-	    hdr = ap_push_array(resp_hdrs);
-	    *(strp++) = '\0';
-	    hdr->field = ap_pstrdup(p, buffer);
-	    while (*strp == ' ' || *strp == '\t')
-		strp++;
-	    hdr->value = ap_pstrdup(p, strp);
-	    gotcr = i;
+	    if (len == 0) /* time to exit the larger loop as well */
+		break;
 	}
     }
-
-    hdr = (struct hdr_entry *) resp_hdrs->elts;
-    for (i = 0; i < resp_hdrs->nelts; i++) {
-	strp = hdr[i].value;
-	j = strlen(strp);
-	while (j > 0 && (strp[j - 1] == ' ' || strp[j - 1] == '\t'))
-	    j--;
-	strp[j] = '\0';
-    }
-
     return resp_hdrs;
 }
 
-long int ap_proxy_send_fb(BUFF *f, request_rec *r, BUFF *f2, struct cache_req *c)
+long int ap_proxy_send_fb(BUFF *f, request_rec *r, cache_req *c)
 {
+    int  ok = 1;
     char buf[IOBUFSIZE];
-    long total_bytes_sent;
+    long total_bytes_rcv;
     register int n, o, w;
     conn_rec *con = r->connection;
+    int alt_to = 1;
 
-    total_bytes_sent = 0;
+    total_bytes_rcv = 0;
+    if (c)
+        c->written = 0;
 
 #ifdef CHARSET_EBCDIC
     /* The cache copy is ASCII, not EBCDIC, even for text/html) */
     ap_bsetflag(f, B_ASCII2EBCDIC|B_EBCDIC2ASCII, 0);
-    if (f2 != NULL)
-	ap_bsetflag(f2, B_ASCII2EBCDIC|B_EBCDIC2ASCII, 0);
+    if (c != NULL && c->fp != NULL)
+	ap_bsetflag(c->fp, B_ASCII2EBCDIC|B_EBCDIC2ASCII, 0);
     ap_bsetflag(con->client, B_ASCII2EBCDIC|B_EBCDIC2ASCII, 0);
 #endif
 
@@ -461,102 +511,105 @@ long int ap_proxy_send_fb(BUFF *f, request_rec *r, BUFF *f2, struct cache_req *c
      * it is unsafe to do a soft_timeout here, at least until the proxy
      * has its own timeout handler which can set both buffers to EOUT.
      */
-    ap_hard_timeout("proxy send body", r);
 
-    while (!con->aborted && f != NULL) {
+    ap_kill_timeout(r);
+
+#ifdef WIN32
+    /* works fine under win32, so leave it */
+    ap_hard_timeout("proxy send body", r);
+    alt_to = 0;
+#else
+    /* CHECKME! Since hard_timeout won't work in unix on sends with partial
+     * cache completion, we have to alternate between hard_timeout
+     * for reads, and soft_timeout for send.  This is because we need
+     * to get a return from ap_bwrite to be able to continue caching.
+     * BUT, if we *can't* continue anyway, just use hard_timeout.
+     */
+
+    if (c) {
+        if (c->len <= 0 || c->cache_completion == 1) {
+            ap_hard_timeout("proxy send body", r);
+            alt_to = 0;
+        }
+    } else {
+        ap_hard_timeout("proxy send body", r);
+        alt_to = 0;
+    }
+#endif
+
+    while (ok) {
+        if (alt_to)
+            ap_hard_timeout("proxy send body", r);
+
+	/* Read block from server */
 	n = ap_bread(f, buf, IOBUFSIZE);
+
+        if (alt_to)
+            ap_kill_timeout(r);
+        else
+            ap_reset_timeout(r);
+
 	if (n == -1) {		/* input error */
-	    if (f2 != NULL)
-		f2 = ap_proxy_cache_error(c);
+	    if (c != NULL)
+		c = ap_proxy_cache_error(c);
 	    break;
 	}
 	if (n == 0)
 	    break;		/* EOF */
 	o = 0;
-	total_bytes_sent += n;
+	total_bytes_rcv += n;
 
-	if (f2 != NULL)
-	    if (ap_bwrite(f2, buf, n) != n)
-		f2 = ap_proxy_cache_error(c);
+	/* Write to cache first. */
+        if (c != NULL && c->fp != NULL) {
+            if (ap_bwrite(c->fp, &buf[0], n) != n) {
+                c = ap_proxy_cache_error(c);
+            } else {
+                c->written += n;
+            }
+        }
 
-	while (n && !con->aborted) {
-	    w = ap_bwrite(con->client, &buf[o], n);
-	    if (w <= 0) {
-		if (f2 != NULL) {
-		    ap_pclosef(c->req->pool, c->fp->fd);
-		    c->fp = NULL;
-		    f2 = NULL;
-		    con->aborted = 1;
-		    unlink(c->tempfile);
-		}
-		break;
-	    }
-	    ap_reset_timeout(r);	/* reset timeout after successful write */
-	    n -= w;
-	    o += w;
-	}
+	/* Write the block to the client, detect aborted transfers */
+        while (n && !con->aborted) {
+            if (alt_to)
+                ap_soft_timeout("proxy send body", r);
+
+            w = ap_bwrite(con->client, &buf[o], n);
+
+            if (alt_to)
+                ap_kill_timeout(r);
+            else
+                ap_reset_timeout(r);
+
+            if (w <= 0) {
+                if (c != NULL) {
+                    /* when a send failure occurs, we need to decide
+                     * whether to continue loading and caching the
+                     * document, or to abort the whole thing
+                     */
+                    ok = (c->len > 0) &&
+                         (c->cache_completion > 0) &&
+                         (c->len * c->cache_completion < total_bytes_rcv);
+
+                    if (! ok) {
+                        ap_pclosef(c->req->pool, c->fp->fd);
+                        c->fp = NULL;
+                        unlink(c->tempfile);
+			c = NULL;
+                    }
+                }
+                con->aborted = 1;
+                break;
+            }
+            n -= w;
+            o += w;
+        }
     }
+
     if (!con->aborted)
 	ap_bflush(con->client);
 
     ap_kill_timeout(r);
-    return total_bytes_sent;
-}
-
-/*
- * Read a header from the array, returning the first entry
- */
-struct hdr_entry *
-          ap_proxy_get_header(array_header *hdrs_arr, const char *name)
-{
-    struct hdr_entry *hdrs;
-    int i;
-
-    hdrs = (struct hdr_entry *) hdrs_arr->elts;
-    for (i = 0; i < hdrs_arr->nelts; i++)
-	if (hdrs[i].field != NULL && strcasecmp(name, hdrs[i].field) == 0)
-	    return &hdrs[i];
-
-    return NULL;
-}
-
-/*
- * Add to the header reply, either concatenating, or replacing existin
- * headers. It stores the pointers provided, so make sure the data
- * is not subsequently overwritten
- */
-struct hdr_entry *
-          ap_proxy_add_header(array_header *hdrs_arr, char *field, char *value,
-			   int rep)
-{
-    int i;
-    struct hdr_entry *hdrs;
-
-    hdrs = (struct hdr_entry *) hdrs_arr->elts;
-    if (rep)
-	for (i = 0; i < hdrs_arr->nelts; i++)
-	    if (hdrs[i].field != NULL && strcasecmp(field, hdrs[i].field) == 0) {
-		hdrs[i].value = value;
-		return hdrs;
-	    }
-
-    hdrs = ap_push_array(hdrs_arr);
-    hdrs->field = field;
-    hdrs->value = value;
-
-    return hdrs;
-}
-
-void ap_proxy_del_header(array_header *hdrs_arr, const char *field)
-{
-    int i;
-    struct hdr_entry *hdrs;
-
-    hdrs = (struct hdr_entry *) hdrs_arr->elts;
-
-    for (i = 0; i < hdrs_arr->nelts; i++)
-	if (hdrs[i].field != NULL && strcasecmp(field, hdrs[i].field) == 0)
-	    hdrs[i].value = NULL;
+    return total_bytes_rcv;
 }
 
 /*
@@ -566,21 +619,22 @@ void ap_proxy_del_header(array_header *hdrs_arr, const char *field)
  * 
  * A timeout should be set before calling this routine.
  */
-void ap_proxy_send_headers(request_rec *r, const char *respline, array_header *hdrs_arr)
+void ap_proxy_send_headers(request_rec *r, const char *respline, table *t)
 {
-    struct hdr_entry *hdrs;
     int i;
     BUFF *fp = r->connection->client;
-
-    hdrs = (struct hdr_entry *) hdrs_arr->elts;
+    table_entry *elts = (table_entry *) ap_table_elts(t)->elts;
 
     ap_bputs(respline, fp);
     ap_bputs(CRLF, fp);
-    for (i = 0; i < hdrs_arr->nelts; i++) {
-	if (hdrs[i].field == NULL)
-	    continue;
-	ap_bvputs(fp, hdrs[i].field, ": ", hdrs[i].value, CRLF, NULL);
-	ap_table_set(r->headers_out, hdrs[i].field, hdrs[i].value);
+
+    for (i = 0; i < ap_table_elts(t)->nelts; ++i) {
+	if (elts[i].key != NULL) {
+	    ap_bvputs(fp, elts[i].key, ": ", elts[i].val, CRLF, NULL);
+	    /* FIXME: @@@ This used to be ap_table_set(), but I think
+	     * ap_table_addn() is correct. MnKr */
+	    ap_table_addn(r->headers_out, elts[i].key, elts[i].val);
+	}
     }
 
     ap_bputs(CRLF, fp);
@@ -606,12 +660,12 @@ int ap_proxy_liststr(const char *list, const char *val)
 	    i = p - list;
 	    do
 		p++;
-	    while (isspace(*p));
+	    while (ap_isspace(*p));
 	}
 	else
 	    i = strlen(list);
 
-	while (i > 0 && isspace(list[i - 1]))
+	while (i > 0 && ap_isspace(list[i - 1]))
 	    i--;
 	if (i == len && strncasecmp(list, val, len) == 0)
 	    return 1;
@@ -734,9 +788,9 @@ int ap_proxy_hex2sec(const char *x)
     for (i = 0, j = 0; i < 8; i++) {
 	ch = x[i];
 	j <<= 4;
-	if (isdigit(ch))
+	if (ap_isdigit(ch))
 	    j |= ch - '0';
-	else if (isupper(ch))
+	else if (ap_isupper(ch))
 	    j |= ch - ('A' - 10);
 	else
 	    j |= ch - ('a' - 10);
@@ -766,10 +820,10 @@ void ap_proxy_sec2hex(int t, char *y)
     y[8] = '\0';
 }
 
-BUFF *
-     ap_proxy_cache_error(struct cache_req *c)
+
+cache_req *ap_proxy_cache_error(cache_req *c)
 {
-    ap_log_error(APLOG_MARK, APLOG_ERR, c->req->server,
+    ap_log_rerror(APLOG_MARK, APLOG_ERR, c->req,
 		 "proxy: error writing to cache file %s", c->tempfile);
     ap_pclosef(c->req->pool, c->fp->fd);
     c->fp = NULL;
@@ -802,7 +856,7 @@ const char *
     static APACHE_TLS char *charpbuf[2];
 
     for (i = 0; host[i] != '\0'; i++)
-	if (!isdigit(host[i]) && host[i] != '.')
+	if (!ap_isdigit(host[i]) && host[i] != '.')
 	    break;
 
     if (host[i] != '\0') {
@@ -828,7 +882,7 @@ const char *
     return NULL;
 }
 
-static char *
+static const char *
      proxy_get_host_of_request(request_rec *r)
 {
     char *url, *user = NULL, *password = NULL, *err, *host;
@@ -847,7 +901,7 @@ static char *
     err = ap_proxy_canon_netloc(r->pool, &url, &user, &password, &host, &port);
 
     if (err != NULL)
-	ap_log_error(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, r,
 		     "%s", err);
 
     r->hostname = host;
@@ -882,7 +936,7 @@ int ap_proxy_is_ipaddr(struct dirconn_entry *This, pool *p)
 	if (*addr == '/' && quads > 0)	/* netmask starts here. */
 	    break;
 
-	if (!isdigit(*addr))
+	if (!ap_isdigit(*addr))
 	    return 0;		/* no digit at start of quad */
 
 	ip_addr[quads] = strtol(addr, &tmp, 0);
@@ -904,7 +958,7 @@ int ap_proxy_is_ipaddr(struct dirconn_entry *This, pool *p)
     for (This->addr.s_addr = 0, i = 0; i < quads; ++i)
 	This->addr.s_addr |= htonl(ip_addr[i] << (24 - 8 * i));
 
-    if (addr[0] == '/' && isdigit(addr[1])) {	/* net mask follows: */
+    if (addr[0] == '/' && ap_isdigit(addr[1])) {	/* net mask follows: */
 	char *tmp;
 
 	++addr;
@@ -1047,7 +1101,7 @@ int ap_proxy_is_domainname(struct dirconn_entry *This, pool *p)
 	return 0;
 
     /* rfc1035 says DNS names must consist of "[-a-zA-Z0-9]" and '.' */
-    for (i = 0; isalnum(addr[i]) || addr[i] == '-' || addr[i] == '.'; ++i)
+    for (i = 0; ap_isalnum(addr[i]) || addr[i] == '-' || addr[i] == '.'; ++i)
 	continue;
 
 #if 0
@@ -1101,7 +1155,7 @@ int ap_proxy_is_hostname(struct dirconn_entry *This, pool *p)
 	return 0;
 
     /* rfc1035 says DNS names must consist of "[-a-zA-Z0-9]" and '.' */
-    for (i = 0; isalnum(addr[i]) || addr[i] == '-' || addr[i] == '.'; ++i);
+    for (i = 0; ap_isalnum(addr[i]) || addr[i] == '-' || addr[i] == '.'; ++i);
 
 #if 0
     if (addr[i] == ':') {
@@ -1127,7 +1181,7 @@ int ap_proxy_is_hostname(struct dirconn_entry *This, pool *p)
 static int proxy_match_hostname(struct dirconn_entry *This, request_rec *r)
 {
     char *host = This->name;
-    char *host2 = proxy_get_host_of_request(r);
+    const char *host2 = proxy_get_host_of_request(r);
     int h2_len;
     int h1_len;
 
@@ -1165,7 +1219,7 @@ int ap_proxy_is_word(struct dirconn_entry *This, pool *p)
 /* Return TRUE if string "str2" occurs literally in "str1" */
 static int proxy_match_word(struct dirconn_entry *This, request_rec *r)
 {
-    char *host = proxy_get_host_of_request(r);
+    const char *host = proxy_get_host_of_request(r);
     return host != NULL && strstr(host, This->name) != NULL;
 }
 
@@ -1182,7 +1236,7 @@ int ap_proxy_doconnect(int sock, struct sockaddr_in *addr, request_rec *r)
 #endif /* WIN32 */
     } while (i == -1 && errno == EINTR);
     if (i == -1) {
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 		     "proxy connect to %s port %d failed",
 		     inet_ntoa(addr->sin_addr), ntohs(addr->sin_port));
     }
@@ -1190,3 +1244,30 @@ int ap_proxy_doconnect(int sock, struct sockaddr_in *addr, request_rec *r)
 
     return i;
 }
+
+/* This function is called by ap_table_do() for all header lines */
+/* (from proxy_http.c and proxy_ftp.c) */
+/* It is passed a table_do_args struct pointer and a MIME field and value pair */
+int ap_proxy_send_hdr_line(void *p, const char *key, const char *value)
+{
+    struct tbl_do_args *parm = (struct tbl_do_args *)p;
+
+    if (key == NULL || value == NULL || value[0] == '\0')
+	return 1;
+    if (!parm->req->assbackwards)
+	ap_rvputs(parm->req, key, ": ", value, CRLF, NULL);
+    if (parm->cache != NULL && parm->cache->fp != NULL &&
+	ap_bvputs(parm->cache->fp, key, ": ", value, CRLF, NULL) == -1)
+	    parm->cache = ap_proxy_cache_error(parm->cache);
+    return 1; /* tell ap_table_do() to continue calling us for more headers */
+}
+
+/* send a text line to one or two BUFF's; return line length */
+unsigned ap_proxy_bputs2(const char *data, BUFF *client, cache_req *cache)
+{
+    unsigned len = ap_bputs(data, client);
+    if (cache != NULL && cache->fp != NULL)
+	ap_bputs(data, cache->fp);
+    return len;
+}
+
