@@ -126,7 +126,7 @@ static char *get_pw(request_rec *r, char *user, char *auth_pwfile)
     const char *rpw, *w;
 
     if (!(f = ap_pcfg_openfile(r->pool, auth_pwfile))) {
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 		    "Could not open password file: %s", auth_pwfile);
 	return NULL;
     }
@@ -201,7 +201,8 @@ static int authenticate_basic_user(request_rec *r)
     auth_config_rec *sec =
     (auth_config_rec *) ap_get_module_config(r->per_dir_config, &auth_module);
     conn_rec *c = r->connection;
-    char *sent_pw, *real_pw;
+    const char *sent_pw;
+    char *real_pw;
     int res;
 
     if ((res = ap_get_basic_auth_pw(r, &sent_pw)))
@@ -213,14 +214,14 @@ static int authenticate_basic_user(request_rec *r)
     if (!(real_pw = get_pw(r, c->user, sec->auth_pwfile))) {
 	if (!(sec->auth_authoritative))
 	    return DECLINED;
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
 		    "user %s not found: %s", c->user, r->uri);
 	ap_note_basic_auth_failure(r);
 	return AUTH_REQUIRED;
     }
     /* anyone know where the prototype for crypt is? */
     if (strcmp(real_pw, (char *) crypt(sent_pw, real_pw))) {
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
 		    "user %s: password mismatch: %s", c->user, r->uri);
 	ap_note_basic_auth_failure(r);
 	return AUTH_REQUIRED;
@@ -240,7 +241,7 @@ static int check_user_access(request_rec *r)
     register int x;
     const char *t, *w;
     table *grpstatus;
-    array_header *reqs_arr = ap_requires(r);
+    const array_header *reqs_arr = ap_requires(r);
     require_line *reqs;
 
     /* BUG FIX: tadc, 11-Nov-1995.  If there is no "requires" directive, 
@@ -263,7 +264,7 @@ static int check_user_access(request_rec *r)
 	method_restricted = 1;
 
 	t = reqs[x].requirement;
-	w = ap_getword(r->pool, &t, ' ');
+	w = ap_getword_white(r->pool, &t);
 	if (!strcmp(w, "valid-user"))
 	    return OK;
 	if (!strcmp(w, "user")) {
@@ -291,6 +292,10 @@ static int check_user_access(request_rec *r)
     if (!(sec->auth_authoritative))
 	return DECLINED;
 
+    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
+	"access to %s failed, reason: user %s not allowed access",
+	r->uri, user);
+	
     ap_note_basic_auth_failure(r);
     return AUTH_REQUIRED;
 }
