@@ -88,7 +88,7 @@ module MODULE_VAR_EXPORT cgi_module;
 
 static int is_scriptaliased(request_rec *r)
 {
-    char *t = ap_table_get(r->notes, "alias-forced-type");
+    const char *t = ap_table_get(r->notes, "alias-forced-type");
     return t && (!strcasecmp(t, "cgi-script"));
 }
 
@@ -169,7 +169,7 @@ static int log_scripterror(request_rec *r, cgi_server_conf * conf, int ret,
     FILE *f;
     struct stat finfo;
 
-    ap_log_error(APLOG_MARK, show_errno|APLOG_ERR, r->server, 
+    ap_log_rerror(APLOG_MARK, show_errno|APLOG_ERR, r, 
 		"%s: %s", error, r->filename);
 
     if (!conf->logname ||
@@ -193,9 +193,9 @@ static int log_scripterror(request_rec *r, cgi_server_conf * conf, int ret,
 }
 
 static int log_script(request_rec *r, cgi_server_conf * conf, int ret,
-		  char *dbuf, char *sbuf, BUFF *script_in, BUFF *script_err)
+		  char *dbuf, const char *sbuf, BUFF *script_in, BUFF *script_err)
 {
-    array_header *hdrs_arr = table_elts(r->headers_in);
+    array_header *hdrs_arr = ap_table_elts(r->headers_in);
     table_entry *hdrs = (table_entry *) hdrs_arr->elts;
     char argsbuffer[HUGE_STRING_LEN];
     FILE *f;
@@ -233,7 +233,7 @@ static int log_script(request_rec *r, cgi_server_conf * conf, int ret,
     }
 
     fputs("%response\n", f);
-    hdrs_arr = table_elts(r->err_headers_out);
+    hdrs_arr = ap_table_elts(r->err_headers_out);
     hdrs = (table_entry *) hdrs_arr->elts;
 
     for (i = 0; i < hdrs_arr->nelts; ++i) {
@@ -289,7 +289,7 @@ static int cgi_child(void *child_stuff, child_info *pinfo)
     int child_pid;
 
 #ifdef DEBUG_CGI
-#ifdef __EMX__
+#ifdef OS2
     /* Under OS/2 need to use device con. */
     FILE *dbg = fopen("con", "w");
 #else
@@ -382,7 +382,7 @@ static int cgi_handler(request_rec *r)
 	return log_scripterror(r, conf, FORBIDDEN, APLOG_NOERRNO,
 			       "attempt to include NPH CGI script");
 
-#if defined(__EMX__) || defined(WIN32)
+#if defined(OS2) || defined(WIN32)
     /* Allow for cgi files without the .EXE extension on them under OS/2 */
     if (r->finfo.st_mode == 0) {
 	struct stat statbuf;
@@ -431,9 +431,9 @@ static int cgi_handler(request_rec *r)
     if (!ap_bspawn_child(r->main ? r->main->pool : r->pool, cgi_child,
 			 (void *) &cld, kill_after_timeout,
 			 &script_out, &script_in, &script_err)) {
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 		    "couldn't spawn child process: %s", r->filename);
-	return SERVER_ERROR;
+	return HTTP_INTERNAL_SERVER_ERROR;
     }
 
     /* Transfer any put/post args, CERN style...
@@ -491,7 +491,8 @@ static int cgi_handler(request_rec *r)
 
     /* Handle script return... */
     if (script_in && !nph) {
-	char *location, sbuf[MAX_STRING_LEN];
+	const char *location;
+	char sbuf[MAX_STRING_LEN];
 	int ret;
 
 	if ((ret = ap_scan_script_header_err_buff(r, script_in, sbuf))) {
