@@ -176,7 +176,7 @@
 module MODULE_VAR_EXPORT config_log_module;
 
 static int xfer_flags = (O_WRONLY | O_APPEND | O_CREAT);
-#if defined(__EMX__) || defined(WIN32)
+#if defined(OS2) || defined(WIN32)
 /* OS/2 dosen't support users and groups */
 static mode_t xfer_mode = (S_IREAD | S_IWRITE);
 #else
@@ -202,10 +202,9 @@ static mode_t xfer_mode = (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
  * multi_log_state is our per-(virtual)-server configuration. We store
  * an array of the logs we are going to use, each of type config_log_state.
  * If a default log format is given by LogFormat, store in default_format
- * (backward compat. with mod_log_config). We also store a pointer to
- * the logs specified for the main server for virtual servers, so that
- * if this vhost has now logs defined, we can use the main server's
- * logs instead.
+ * (backward compat. with mod_log_config).  We also store for each virtual
+ * server a pointer to the logs specified for the main server, so that if this
+ * vhost has no logs defined, we can use the main server's logs instead.
  *
  * So, for the main server, config_logs contains a list of the log files
  * and server_config_logs in empty. For a vhost, server_config_logs
@@ -215,6 +214,7 @@ static mode_t xfer_mode = (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
  */
 
 typedef struct {
+    char *default_format_string;
     array_header *default_format;
     array_header *config_logs;
     array_header *server_config_logs;
@@ -232,6 +232,7 @@ typedef struct {
 
 typedef struct {
     char *fname;
+    char *format_string;
     array_header *format;
     int log_fd;
 #ifdef BUFFERED_LOGS
@@ -242,9 +243,10 @@ typedef struct {
 
 /*
  * Format items...
+ * Note that many of these could have ap_sprintfs replaced with static buffers.
  */
 
-typedef char *(*item_key_func) (request_rec *, char *);
+typedef const char *(*item_key_func) (request_rec *, char *);
 
 typedef struct {
     item_key_func func;
@@ -269,28 +271,28 @@ static char *pfmt(pool *p, int i)
     }
 }
 
-static char *constant_item(request_rec *dummy, char *stuff)
+static const char *constant_item(request_rec *dummy, char *stuff)
 {
     return stuff;
 }
 
-static char *log_remote_host(request_rec *r, char *a)
+static const char *log_remote_host(request_rec *r, char *a)
 {
-    return (char *) ap_get_remote_host(r->connection, r->per_dir_config,
+    return ap_get_remote_host(r->connection, r->per_dir_config,
                                     REMOTE_NAME);
 }
 
-static char *log_remote_address(request_rec *r, char *a)
+static const char *log_remote_address(request_rec *r, char *a)
 {
     return r->connection->remote_ip;
 }
 
-static char *log_remote_logname(request_rec *r, char *a)
+static const char *log_remote_logname(request_rec *r, char *a)
 {
-    return (char *) ap_get_remote_logname(r);
+    return ap_get_remote_logname(r);
 }
 
-static char *log_remote_user(request_rec *r, char *a)
+static const char *log_remote_user(request_rec *r, char *a)
 {
     char *rvalue = r->connection->user;
 
@@ -303,7 +305,7 @@ static char *log_remote_user(request_rec *r, char *a)
     return rvalue;
 }
 
-static char *log_request_line(request_rec *r, char *a)
+static const char *log_request_line(request_rec *r, char *a)
 {
 	    /* NOTE: If the original request contained a password, we
 	     * re-write the request line here to contain XXXXXX instead:
@@ -316,20 +318,20 @@ static char *log_request_line(request_rec *r, char *a)
 					: r->the_request;
 }
 
-static char *log_request_file(request_rec *r, char *a)
+static const char *log_request_file(request_rec *r, char *a)
 {
     return r->filename;
 }
-static char *log_request_uri(request_rec *r, char *a)
+static const char *log_request_uri(request_rec *r, char *a)
 {
     return r->uri;
 }
-static char *log_status(request_rec *r, char *a)
+static const char *log_status(request_rec *r, char *a)
 {
     return pfmt(r->pool, r->status);
 }
 
-static char *log_bytes_sent(request_rec *r, char *a)
+static const char *log_bytes_sent(request_rec *r, char *a)
 {
     if (!r->sent_bodyct) {
         return "-";
@@ -341,14 +343,14 @@ static char *log_bytes_sent(request_rec *r, char *a)
     }
 }
 
-static char *log_header_in(request_rec *r, char *a)
+static const char *log_header_in(request_rec *r, char *a)
 {
     return ap_table_get(r->headers_in, a);
 }
 
-static char *log_header_out(request_rec *r, char *a)
+static const char *log_header_out(request_rec *r, char *a)
 {
-    char *cp = ap_table_get(r->headers_out, a);
+    const char *cp = ap_table_get(r->headers_out, a);
     if (!strcasecmp(a, "Content-type") && r->content_type) {
         cp = r->content_type;
     }
@@ -358,16 +360,16 @@ static char *log_header_out(request_rec *r, char *a)
     return ap_table_get(r->err_headers_out, a);
 }
 
-static char *log_note(request_rec *r, char *a)
+static const char *log_note(request_rec *r, char *a)
 {
     return ap_table_get(r->notes, a);
 }
-static char *log_env_var(request_rec *r, char *a)
+static const char *log_env_var(request_rec *r, char *a)
 {
     return ap_table_get(r->subprocess_env, a);
 }
 
-static char *log_request_time(request_rec *r, char *a)
+static const char *log_request_time(request_rec *r, char *a)
 {
     int timz;
     struct tm *t;
@@ -395,7 +397,7 @@ static char *log_request_time(request_rec *r, char *a)
     return ap_pstrdup(r->pool, tstr);
 }
 
-static char *log_request_duration(request_rec *r, char *a)
+static const char *log_request_duration(request_rec *r, char *a)
 {
     return ap_psprintf(r->pool, "%ld", time(NULL) - r->request_time);
 }
@@ -403,17 +405,17 @@ static char *log_request_duration(request_rec *r, char *a)
 /* These next two routines use the canonical name:port so that log
  * parsers don't need to duplicate all the vhost parsing crud.
  */
-static char *log_virtual_host(request_rec *r, char *a)
+static const char *log_virtual_host(request_rec *r, char *a)
 {
-    return ap_pstrdup(r->pool, ap_get_server_name(r));
+    return ap_get_server_name(r);
 }
 
-static char *log_server_port(request_rec *r, char *a)
+static const char *log_server_port(request_rec *r, char *a)
 {
     return ap_psprintf(r->pool, "%u", ap_get_server_port(r));
 }
 
-static char *log_child_pid(request_rec *r, char *a)
+static const char *log_child_pid(request_rec *r, char *a)
 {
     return ap_psprintf(r->pool, "%ld", (long) getpid());
 }
@@ -581,7 +583,7 @@ static char *parse_log_item(pool *p, log_format_item *it, const char **sa)
         case '8':
         case '9':
             i = *s - '0';
-            while (isdigit(*++s)) {
+            while (ap_isdigit(*++s)) {
                 i = i * 10 + (*s) - '0';
             }
             if (!it->conditions) {
@@ -634,10 +636,10 @@ static array_header *parse_log_string(pool *p, const char *s, const char **err)
  * Actually logging.
  */
 
-static char *process_item(request_rec *r, request_rec *orig,
+static const char *process_item(request_rec *r, request_rec *orig,
                           log_format_item *item)
 {
-    char *cp;
+    const char *cp;
 
     /* First, see if we need to process this thing at all... */
 
@@ -679,7 +681,8 @@ static int config_log_transaction(request_rec *r, config_log_state *cls,
                                   array_header *default_format)
 {
     log_format_item *items;
-    char *str, **strs, *s;
+    char *str, *s;
+    const char **strs;
     int *strl;
     request_rec *orig;
     int i;
@@ -782,6 +785,7 @@ static void *make_config_log_state(pool *p, server_rec *s)
     multi_log_state *mls = (multi_log_state *) ap_palloc(p, sizeof(multi_log_state));
 
     mls->config_logs = ap_make_array(p, 1, sizeof(config_log_state));
+    mls->default_format_string = NULL;
     mls->default_format = NULL;
     mls->server_config_logs = NULL;
     mls->formats = ap_make_table(p, 4);
@@ -803,6 +807,7 @@ static void *merge_config_log_state(pool *p, void *basev, void *addv)
 
     add->server_config_logs = base->config_logs;
     if (!add->default_format) {
+        add->default_format_string = base->default_format_string;
         add->default_format = base->default_format;
     }
     add->formats = ap_overlay_tables(p, base->formats, add->formats);
@@ -817,7 +822,6 @@ static const char *log_format(cmd_parms *cmd, void *dummy, char *fmt,
                               char *name)
 {
     const char *err_string = NULL;
-    char *format;
     multi_log_state *mls = ap_get_module_config(cmd->server->module_config,
                                              &config_log_module);
 
@@ -833,14 +837,8 @@ static const char *log_format(cmd_parms *cmd, void *dummy, char *fmt,
         }
     }
     else {
-        /*
-         * See if we were given a name rather than a format string.
-         */
-        format = ap_table_get(mls->formats, fmt);
-        if (format == NULL) {
-            format = fmt;
-        }
-        mls->default_format = parse_log_string(cmd->pool, format, &err_string);
+        mls->default_format_string = fmt;
+        mls->default_format = parse_log_string(cmd->pool, fmt, &err_string);
     }
     return err_string;
 }
@@ -852,17 +850,15 @@ static const char *add_custom_log(cmd_parms *cmd, void *dummy, char *fn,
     multi_log_state *mls = ap_get_module_config(cmd->server->module_config,
                                              &config_log_module);
     config_log_state *cls;
-    char *format;
 
     cls = (config_log_state *) ap_push_array(mls->config_logs);
     cls->fname = fn;
+    cls->format_string = fmt;
     if (!fmt) {
         cls->format = NULL;
     }
     else {
-        format = ap_table_get(mls->formats, fmt);
-        format = (format != NULL) ? format : fmt;
-        cls->format = parse_log_string(cmd->pool, format, &err_string);
+        cls->format = parse_log_string(cmd->pool, fmt, &err_string);
     }
     cls->log_fd = -1;
 
@@ -935,6 +931,14 @@ static config_log_state *open_multi_logs(server_rec *s, pool *p)
                                              &config_log_module);
     config_log_state *clsarray;
     const char *dummy;
+    const char *format;
+
+    if (mls->default_format_string) {
+	format = ap_table_get(mls->formats, mls->default_format_string);
+	if (format) {
+	    mls->default_format = parse_log_string(p, format, &dummy);
+	}
+    }    
 
     if (!mls->default_format) {
         mls->default_format = parse_log_string(p, DEFAULT_LOG_FORMAT, &dummy);
@@ -945,6 +949,13 @@ static config_log_state *open_multi_logs(server_rec *s, pool *p)
         for (i = 0; i < mls->config_logs->nelts; ++i) {
             config_log_state *cls = &clsarray[i];
 
+	    if (cls->format_string) {
+		format = ap_table_get(mls->formats, cls->format_string);
+		if (format) {
+		    cls->format = parse_log_string(p, format, &dummy);
+		}
+	    }
+
             cls = open_config_log(s, p, cls, mls->default_format);
         }
     }
@@ -953,6 +964,13 @@ static config_log_state *open_multi_logs(server_rec *s, pool *p)
         for (i = 0; i < mls->server_config_logs->nelts; ++i) {
             config_log_state *cls = &clsarray[i];
 
+	    if (cls->format_string) {
+		format = ap_table_get(mls->formats, cls->format_string);
+		if (format) {
+		    cls->format = parse_log_string(p, format, &dummy);
+		}
+	    }
+
             cls = open_config_log(s, p, cls, mls->default_format);
         }
     }
