@@ -199,7 +199,7 @@ static const command_rec mime_cmds[] =
  * equivalent to 27 in ASCII, and makes it work in EBCDIC.
  */
 #define MIME_HASHSIZE ('z'-'a'+2)
-#define hash(i) (isalpha(i) ? (tolower(i)) - 'a' : (MIME_HASHSIZE-1))
+#define hash(i) (ap_isalpha(i) ? (ap_tolower(i)) - 'a' : (MIME_HASHSIZE-1))
 
 static table *hash_buckets[MIME_HASHSIZE];
 
@@ -245,7 +245,9 @@ static int find_ct(request_rec *r)
     const char *fn = strrchr(r->filename, '/');
     mime_dir_config *conf =
     (mime_dir_config *) ap_get_module_config(r->per_dir_config, &mime_module);
-    char *ext, *type, *orighandler = r->handler;
+    char *ext;
+    const char *orighandler = r->handler;
+    const char *type;
 
     if (S_ISDIR(r->finfo.st_mode)) {
         r->content_type = DIR_MAGIC_TYPE;
@@ -273,12 +275,12 @@ static int find_ct(request_rec *r)
 
         /* Check for Content-Language */
         if ((type = ap_table_get(conf->language_types, ext))) {
-            char **new;
+            const char **new;
 
             r->content_language = type;         /* back compat. only */
             if (!r->content_languages)
                 r->content_languages = ap_make_array(r->pool, 2, sizeof(char *));
-            new = (char **) ap_push_array(r->content_languages);
+            new = (const char **) ap_push_array(r->content_languages);
             *new = type;
             found = 1;
         }
