@@ -575,7 +575,7 @@ static int magic_rsl_add(request_rec *r, char *str)
 
     /* make sure we have a list to put it in */
     if (!req_dat) {
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 		    MODNAME ": request config should not be NULL");
 	if (!(req_dat = magic_set_config(r))) {
 	    /* failure */
@@ -677,7 +677,7 @@ static char *rsl_strdup(request_rec *r, int start_frag, int start_pos, int len)
     /* clean up and return */
     result[res_pos] = 0;
 #if MIME_MAGIC_DEBUG
-    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 	     MODNAME ": rsl_strdup() %d chars: %s", res_pos - 1, result);
 #endif
     return result;
@@ -721,7 +721,7 @@ static int magic_rsl_to_request(request_rec *r)
 	 frag = frag->next, cur_frag++) {
 	/* loop through the characters in the fragment */
 	for (cur_pos = 0; frag->str[cur_pos]; cur_pos++) {
-	    if (isspace(frag->str[cur_pos])) {
+	    if (ap_isspace(frag->str[cur_pos])) {
 		/* process whitespace actions for each state */
 		if (state == rsl_leading_space) {
 		    /* eat whitespace in this state */
@@ -749,7 +749,7 @@ static int magic_rsl_to_request(request_rec *r)
 		else {
 		    /* should not be possible */
 		    /* abandon malfunctioning module */
-		    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+		    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 				MODNAME ": bad state %d (ws)", state);
 		    return DECLINED;
 		}
@@ -793,7 +793,7 @@ static int magic_rsl_to_request(request_rec *r)
 		else {
 		    /* should not be possible */
 		    /* abandon malfunctioning module */
-		    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+		    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 				MODNAME ": bad state %d (ns)", state);
 		    return DECLINED;
 		}
@@ -813,17 +813,21 @@ static int magic_rsl_to_request(request_rec *r)
     /* save the info in the request record */
     if (state == rsl_subtype || state == rsl_encoding ||
 	state == rsl_encoding) {
-	r->content_type = rsl_strdup(r, type_frag, type_pos, type_len);
+        char *tmp;
+	tmp = rsl_strdup(r, type_frag, type_pos, type_len);
 	/* XXX: this could be done at config time I'm sure... but I'm
 	 * confused by all this magic_rsl stuff. -djg */
-	ap_str_tolower(r->content_type);
+	ap_content_type_tolower(tmp);
+	r->content_type = tmp;
     }
     if (state == rsl_encoding) {
-	r->content_encoding = rsl_strdup(r, encoding_frag,
+        char *tmp;
+	tmp = rsl_strdup(r, encoding_frag,
 					 encoding_pos, encoding_len);
 	/* XXX: this could be done at config time I'm sure... but I'm
 	 * confused by all this magic_rsl stuff. -djg */
-	ap_str_tolower(r->content_encoding);
+	ap_str_tolower(tmp);
+	r->content_encoding = tmp;
     }
 
     /* detect memory allocation errors */
@@ -864,7 +868,7 @@ static int magic_process(request_rec *r)
 
     if ((fd = ap_popenf(r->pool, r->filename, O_RDONLY, 0)) < 0) {
 	/* We can't open it, but we were able to stat it. */
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 		    MODNAME ": can't read `%s'", r->filename);
 	/* let some other handler decide what the problem is */
 	return DECLINED;
@@ -874,7 +878,7 @@ static int magic_process(request_rec *r)
      * try looking at the first HOWMANY bytes
      */
     if ((nbytes = read(fd, (char *) buf, sizeof(buf) - 1)) == -1) {
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 		    MODNAME ": read failed: %s", r->filename);
 	return HTTP_INTERNAL_SERVER_ERROR;
     }
@@ -919,8 +923,7 @@ static void tryit(request_rec *r, unsigned char *buf, int nb)
     magic_rsl_puts(r, MIME_BINARY_UNKNOWN);
 }
 
-#define    EATAB {while (isascii((unsigned char) *l) && \
-              isspace((unsigned char) *l))  ++l;}
+#define    EATAB {while (ap_isspace((unsigned char) *l))  ++l;}
 
 /*
  * apprentice - load configuration from the magic file r
@@ -963,7 +966,7 @@ static int apprentice(server_rec *s, pool *p)
 
 	/* skip leading whitespace */
 	ws_offset = 0;
-	while (line[ws_offset] && isspace(line[ws_offset])) {
+	while (line[ws_offset] && ap_isspace(line[ws_offset])) {
 	    ws_offset++;
 	}
 
@@ -1006,10 +1009,10 @@ static int apprentice(server_rec *s, pool *p)
     ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, s,
 		MODNAME ": apprentice test");
     for (m = conf->magic; m; m = m->next) {
-	if (isprint((((unsigned long) m) >> 24) & 255) &&
-	    isprint((((unsigned long) m) >> 16) & 255) &&
-	    isprint((((unsigned long) m) >> 8) & 255) &&
-	    isprint(((unsigned long) m) & 255)) {
+	if (ap_isprint((((unsigned long) m) >> 24) & 255) &&
+	    ap_isprint((((unsigned long) m) >> 16) & 255) &&
+	    ap_isprint((((unsigned long) m) >> 8) & 255) &&
+	    ap_isprint(((unsigned long) m) & 255)) {
 	    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, s,
 			MODNAME ": apprentice: POINTER CLOBBERED! "
 			"m=\"%c%c%c%c\" line=%d",
@@ -1137,7 +1140,7 @@ static int parse(server_rec *serv, pool *p, char *l, int lineno)
 	s = l;
 	if (*l == '+' || *l == '-')
 	    l++;
-	if (isdigit((unsigned char) *l)) {
+	if (ap_isdigit((unsigned char) *l)) {
 	    m->in.offset = strtol(l, &t, 0);
 	    if (*s == '-')
 		m->in.offset = -m->in.offset;
@@ -1152,7 +1155,7 @@ static int parse(server_rec *serv, pool *p, char *l, int lineno)
     }
 
 
-    while (isascii((unsigned char) *l) && isdigit((unsigned char) *l))
+    while (ap_isdigit((unsigned char) *l))
 	++l;
     EATAB;
 
@@ -1250,8 +1253,7 @@ static int parse(server_rec *serv, pool *p, char *l, int lineno)
 	}
 	/* FALL THROUGH */
     default:
-	if (*l == 'x' && isascii((unsigned char) l[1]) &&
-	    isspace((unsigned char) l[1])) {
+	if (*l == 'x' && ap_isspace((unsigned char) l[1])) {
 	    m->reln = *l;
 	    ++l;
 	    goto GetDesc;	/* Bill The Cat */
@@ -1323,7 +1325,7 @@ static char *getstr(server_rec *serv, register char *s, register char *p,
     register int val;
 
     while ((c = *s++) != '\0') {
-	if (isspace((unsigned char) c))
+	if (ap_isspace((unsigned char) c))
 	    break;
 	if (p >= pmax) {
 	    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, serv,
@@ -1426,9 +1428,7 @@ static char *getstr(server_rec *serv, register char *s, register char *p,
 /* Single hex char to int; -1 if not a hex char. */
 static int hextoint(int c)
 {
-    if (!isascii((unsigned char) c))
-	return -1;
-    if (isdigit((unsigned char) c))
+    if (ap_isdigit((unsigned char) c))
 	return c - '0';
     if ((c >= 'a') && (c <= 'f'))
 	return c + 10 - 'a';
@@ -1479,7 +1479,7 @@ static int fsmagic(request_rec *r, const char *fn)
 	/* We used stat(), the only possible reason for this is that the
 	 * symlink is broken.
 	 */
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 		    MODNAME ": broken symlink (%s)", fn);
 	return HTTP_INTERNAL_SERVER_ERROR;
 #endif
@@ -1493,7 +1493,7 @@ static int fsmagic(request_rec *r, const char *fn)
     case S_IFREG:
 	break;
     default:
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 		    MODNAME ": invalid mode 0%o.", (unsigned int)r->finfo.st_mode);
 	return HTTP_INTERNAL_SERVER_ERROR;
     }
@@ -1560,7 +1560,7 @@ static int match(request_rec *r, unsigned char *s, int nbytes)
     struct magic *m;
 
 #if MIME_MAGIC_DEBUG
-    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		MODNAME ": match conf=%x file=%s m=%s m->next=%s last=%s",
 		conf,
 		conf->magicfile ? conf->magicfile : "NULL",
@@ -1571,11 +1571,11 @@ static int match(request_rec *r, unsigned char *s, int nbytes)
 
 #if MIME_MAGIC_DEBUG
     for (m = conf->magic; m; m = m->next) {
-	if (isprint((((unsigned long) m) >> 24) & 255) &&
-	    isprint((((unsigned long) m) >> 16) & 255) &&
-	    isprint((((unsigned long) m) >> 8) & 255) &&
-	    isprint(((unsigned long) m) & 255)) {
-	    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	if (ap_isprint((((unsigned long) m) >> 24) & 255) &&
+	    ap_isprint((((unsigned long) m) >> 16) & 255) &&
+	    ap_isprint((((unsigned long) m) >> 8) & 255) &&
+	    ap_isprint(((unsigned long) m) & 255)) {
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 			MODNAME ": match: POINTER CLOBBERED! "
 			"m=\"%c%c%c%c\"",
 			(((unsigned long) m) >> 24) & 255,
@@ -1590,7 +1590,7 @@ static int match(request_rec *r, unsigned char *s, int nbytes)
     for (m = conf->magic; m; m = m->next) {
 #if MIME_MAGIC_DEBUG
 	rule_counter++;
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		    MODNAME ": line=%d desc=%s", m->lineno, m->desc);
 #endif
 
@@ -1610,7 +1610,7 @@ static int match(request_rec *r, unsigned char *s, int nbytes)
 	    while (m_cont && (m_cont->cont_level != 0)) {
 #if MIME_MAGIC_DEBUG
 		rule_counter++;
-		ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+		ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 			MODNAME ": line=%d mc=%x mc->next=%x cont=%d desc=%s",
 			    m_cont->lineno, m_cont,
 			    m_cont->next, m_cont->cont_level,
@@ -1629,7 +1629,7 @@ static int match(request_rec *r, unsigned char *s, int nbytes)
 	/* if we get here, the main entry rule was a match */
 	/* this will be the last run through the loop */
 #if MIME_MAGIC_DEBUG
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		    MODNAME ": rule matched, line=%d type=%d %s",
 		    m->lineno, m->type,
 		    (m->type == STRING) ? m->value.s : "");
@@ -1653,7 +1653,7 @@ static int match(request_rec *r, unsigned char *s, int nbytes)
 	m = m->next;
 	while (m && (m->cont_level != 0)) {
 #if MIME_MAGIC_DEBUG
-	    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 			MODNAME ": match line=%d cont=%d type=%d %s",
 			m->lineno, m->cont_level, m->type,
 			(m->type == STRING) ? m->value.s : "");
@@ -1697,13 +1697,13 @@ static int match(request_rec *r, unsigned char *s, int nbytes)
 	    m = m->next;
 	}
 #if MIME_MAGIC_DEBUG
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		    MODNAME ": matched after %d rules", rule_counter);
 #endif
 	return 1;		/* all through */
     }
 #if MIME_MAGIC_DEBUG
-    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		MODNAME ": failed after %d rules", rule_counter);
 #endif
     return 0;			/* no match at all */
@@ -1750,7 +1750,7 @@ static void mprint(request_rec *r, union VALUETYPE *p, struct magic *m)
 	(void) magic_rsl_printf(r, m->desc, pp);
 	return;
     default:
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 		    MODNAME ": invalid m->type (%d) in mprint().",
 		    m->type);
 	return;
@@ -1796,7 +1796,7 @@ static int mconvert(request_rec *r, union VALUETYPE *p, struct magic *m)
 	    ((p->hl[3] << 24) | (p->hl[2] << 16) | (p->hl[1] << 8) | (p->hl[0]));
 	return 1;
     default:
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 		    MODNAME ": invalid type %d in mconvert().", m->type);
 	return 0;
     }
@@ -1848,7 +1848,7 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
     int matched;
 
     if ((m->value.s[0] == 'x') && (m->value.s[1] == '\0')) {
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 		    MODNAME ": BOINK");
 	return 1;
     }
@@ -1893,7 +1893,7 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
 	break;
     default:
 	/*  bogosity, pretend that it just wasn't a match */
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 		    MODNAME ": invalid type %d in mcheck().", m->type);
 	return 0;
     }
@@ -1903,7 +1903,7 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
     switch (m->reln) {
     case 'x':
 #if MIME_MAGIC_DEBUG
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		    "%lu == *any* = 1", v);
 #endif
 	matched = 1;
@@ -1912,7 +1912,7 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
     case '!':
 	matched = v != l;
 #if MIME_MAGIC_DEBUG
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		    "%lu != %lu = %d", v, l, matched);
 #endif
 	break;
@@ -1920,7 +1920,7 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
     case '=':
 	matched = v == l;
 #if MIME_MAGIC_DEBUG
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		    "%lu == %lu = %d", v, l, matched);
 #endif
 	break;
@@ -1929,14 +1929,14 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
 	if (m->flag & UNSIGNED) {
 	    matched = v > l;
 #if MIME_MAGIC_DEBUG
-	    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 			"%lu > %lu = %d", v, l, matched);
 #endif
 	}
 	else {
 	    matched = (long) v > (long) l;
 #if MIME_MAGIC_DEBUG
-	    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 			"%ld > %ld = %d", v, l, matched);
 #endif
 	}
@@ -1946,14 +1946,14 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
 	if (m->flag & UNSIGNED) {
 	    matched = v < l;
 #if MIME_MAGIC_DEBUG
-	    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 			"%lu < %lu = %d", v, l, matched);
 #endif
 	}
 	else {
 	    matched = (long) v < (long) l;
 #if MIME_MAGIC_DEBUG
-	    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 			"%ld < %ld = %d", v, l, matched);
 #endif
 	}
@@ -1962,7 +1962,7 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
     case '&':
 	matched = (v & l) == l;
 #if MIME_MAGIC_DEBUG
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		    "((%lx & %lx) == %lx) = %d", v, l, l, matched);
 #endif
 	break;
@@ -1970,7 +1970,7 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
     case '^':
 	matched = (v & l) != l;
 #if MIME_MAGIC_DEBUG
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		    "((%lx & %lx) != %lx) = %d", v, l, l, matched);
 #endif
 	break;
@@ -1978,7 +1978,7 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
     default:
 	/* bogosity, pretend it didn't match */
 	matched = 0;
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_ERR, r,
 		    MODNAME ": mcheck: can't happen: invalid relation %d.",
 		    m->reln);
 	break;
@@ -1992,7 +1992,7 @@ static int mcheck(request_rec *r, union VALUETYPE *p, struct magic *m)
 
 static int ascmagic(request_rec *r, unsigned char *buf, int nbytes)
 {
-    int i, has_escapes = 0;
+    int has_escapes = 0;
     unsigned char *s;
     char nbuf[HOWMANY + 1];	/* one extra for terminating '\0' */
     char *token;
@@ -2009,16 +2009,15 @@ static int ascmagic(request_rec *r, unsigned char *buf, int nbytes)
     if (*buf == '.') {
 	unsigned char *tp = buf + 1;
 
-	while (isascii(*tp) && isspace(*tp))
+	while (ap_isspace(*tp))
 	    ++tp;		/* skip leading whitespace */
-	if ((isascii(*tp) && (isalnum(*tp) || *tp == '\\') &&
-	     isascii(*(tp + 1)) && (isalnum(*(tp + 1)) || *tp == '"'))) {
+	if ((ap_isalnum(*tp) || *tp == '\\') &&
+	     (ap_isalnum(*(tp + 1)) || *tp == '"')) {
 	    magic_rsl_puts(r, "application/x-troff");
 	    return 1;
 	}
     }
-    if ((*buf == 'c' || *buf == 'C') &&
-	isascii(*(buf + 1)) && isspace(*(buf + 1))) {
+    if ((*buf == 'c' || *buf == 'C') && ap_isspace(*(buf + 1))) {
 	/* Fortran */
 	magic_rsl_puts(r, "text/plain");
 	return 1;
@@ -2055,11 +2054,6 @@ static int ascmagic(request_rec *r, unsigned char *buf, int nbytes)
 	return 1;
     }
 
-    for (i = 0; i < nbytes; i++) {
-	if (!isascii(*(buf + i)))
-	    return 0;		/* not all ascii */
-    }
-
     /* all else fails, but it is ascii... */
     if (has_escapes) {
 	/* text with escape sequences */
@@ -2159,7 +2153,7 @@ static int uncompress_child(void *data, child_info *pinfo)
     return (child_pid);
 #else
     execvp(compr[parm->method].argv[0], compr[parm->method].argv);
-    ap_log_error(APLOG_MARK, APLOG_ERR, parm->r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_ERR, parm->r,
 		MODNAME ": could not execute `%s'.",
 		compr[parm->method].argv[0]);
     return -1;
@@ -2185,14 +2179,14 @@ static int uncompress(request_rec *r, int method, const unsigned char *old,
 
     if (!ap_bspawn_child(sub_pool, uncompress_child, &parm, kill_always,
 			 &bin, &bout, NULL)) {
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 		    MODNAME ": couldn't spawn uncompress process: %s", r->uri);
 	return -1;
     }
 
     if (ap_bwrite(bin, old, n) != n) {
 	ap_destroy_pool(sub_pool);
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 		    MODNAME ": write failed.");
 	return -1;
     }
@@ -2200,7 +2194,7 @@ static int uncompress(request_rec *r, int method, const unsigned char *old,
     *newch = (unsigned char *) ap_palloc(r->pool, n);
     if ((n = ap_bread(bout, *newch, n)) <= 0) {
 	ap_destroy_pool(sub_pool);
-	ap_log_error(APLOG_MARK, APLOG_ERR, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
 	    MODNAME ": read failed %s", r->filename);
 	return -1;
     }
@@ -2273,7 +2267,7 @@ static long from_oct(int digs, char *where)
 {
     register long value;
 
-    while (isspace(*where)) {	/* Skip spaces */
+    while (ap_isspace(*where)) {	/* Skip spaces */
 	where++;
 	if (--digs <= 0)
 	    return -1;		/* All blank field */
@@ -2284,7 +2278,7 @@ static long from_oct(int digs, char *where)
 	--digs;
     }
 
-    if (digs > 0 && *where && !isspace(*where))
+    if (digs > 0 && *where && !ap_isspace(*where))
 	return -1;		/* Ended on non-space/nul */
 
     return value;
@@ -2307,16 +2301,16 @@ static int revision_suffix(request_rec *r)
     request_rec *sub;
 
 #if MIME_MAGIC_DEBUG
-    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		MODNAME ": revision_suffix checking %s", r->filename);
 #endif /* MIME_MAGIC_DEBUG */
 
     /* check for recognized revision suffix */
     suffix_pos = strlen(r->filename) - 1;
-    if (!isdigit(r->filename[suffix_pos])) {
+    if (!ap_isdigit(r->filename[suffix_pos])) {
 	return 0;
     }
-    while (suffix_pos >= 0 && isdigit(r->filename[suffix_pos]))
+    while (suffix_pos >= 0 && ap_isdigit(r->filename[suffix_pos]))
 	suffix_pos--;
     if (suffix_pos < 0 || r->filename[suffix_pos] != '@') {
 	return 0;
@@ -2326,7 +2320,7 @@ static int revision_suffix(request_rec *r)
     result = 0;
     sub_filename = ap_pstrndup(r->pool, r->filename, suffix_pos);
 #if MIME_MAGIC_DEBUG
-    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		MODNAME ": subrequest lookup for %s", sub_filename);
 #endif /* MIME_MAGIC_DEBUG */
     sub = ap_sub_req_lookup_file(sub_filename, r);
@@ -2335,7 +2329,7 @@ static int revision_suffix(request_rec *r)
     if (sub->content_type) {
 	r->content_type = ap_pstrdup(r->pool, sub->content_type);
 #if MIME_MAGIC_DEBUG
-	ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r->server,
+	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, r,
 		    MODNAME ": subrequest %s got %s",
 		    sub_filename, r->content_type);
 #endif /* MIME_MAGIC_DEBUG */
@@ -2384,10 +2378,10 @@ static void magic_init(server_rec *main_server, pool *p)
 	    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, s,
 			MODNAME ": magic_init 1 test");
 	    for (m = conf->magic; m; m = m->next) {
-		if (isprint((((unsigned long) m) >> 24) & 255) &&
-		    isprint((((unsigned long) m) >> 16) & 255) &&
-		    isprint((((unsigned long) m) >> 8) & 255) &&
-		    isprint(((unsigned long) m) & 255)) {
+		if (ap_isprint((((unsigned long) m) >> 24) & 255) &&
+		    ap_isprint((((unsigned long) m) >> 16) & 255) &&
+		    ap_isprint((((unsigned long) m) >> 8) & 255) &&
+		    ap_isprint(((unsigned long) m) & 255)) {
 		    ap_log_error(APLOG_MARK, APLOG_NOERRNO | APLOG_DEBUG, s,
 				MODNAME ": magic_init 1: POINTER CLOBBERED! "
 				"m=\"%c%c%c%c\" line=%d",
