@@ -107,11 +107,19 @@
     /* Include from the Apache server ... */
 #include "httpd.h"
 #include "http_config.h"
+#include "http_conf_globals.h"
 #include "http_request.h"
 #include "http_core.h"
 #include "http_log.h"
 #include "http_vhost.h"
 
+    /*
+     * The key in the r->notes table wherein we store our accumulated
+     * Vary values, and the one used for per-condition checks in a chain.
+     */
+#define VARY_KEY "rewrite-Vary"
+#define VARY_KEY_THIS "rewrite-Vary-this"
+
     /* The NDBM support:
      * We support only NDBM files.
      * But we have to stat the file for the mtime,
@@ -129,7 +137,7 @@
 
     /* The locking support:
      * Try to determine whether we should use fcntl() or flock().
-     * Would be better conf.h could provide this... :-(
+     * Would be better ap_config.h could provide this... :-(
      */
 #if defined(USE_FCNTL_SERIALIZED_ACCEPT)
 #define USE_FCNTL 1
@@ -228,7 +236,7 @@
 #define LONG_STRING_LEN 2048
 #endif
 
-#define MAX_ENV_FLAGS 5
+#define MAX_ENV_FLAGS 15
 
 #define MAX_NMATCH    10
 
