@@ -153,6 +153,10 @@ static void *so_sconf_create(pool *p, server_rec *s)
     soc = (so_server_conf *)ap_pcalloc(p, sizeof(so_server_conf));
     soc->loaded_modules = ap_make_array(p, DYNAMIC_MODULE_LIMIT, 
                                      sizeof(moduleinfo));
+#ifndef NO_DLOPEN
+    ap_os_dso_init();
+#endif
+
     return (void *)soc;
 }
 
