@@ -57,13 +57,12 @@
 
 
 #ifdef CHARSET_EBCDIC
-#include "conf.h"
+#include "ap_config.h"
 #include "ebcdic.h"
 /*
 	   Initial Port for  Apache-1.3 by <Martin.Kraemer@Mch.SNI.De>
 
-"BS2000 OSD" is a POSIX on a main frame.
-It is made by Siemens Nixdorf AG, Germany.
+"BS2000 OSD" is a POSIX on a main frame. It is made by Siemens AG, Germany.
 Within the POSIX subsystem, the same character set was chosen as in
 "native BS2000", namely EBCDIC.
 
