@@ -22,13 +22,13 @@
 #ifndef STRICT
  #define STRICT
 #endif
-#define NO_UNISTD_H
+#define CASE_BLIND_FILESYSTEM
 #define NO_WRITEV
 #define NO_SETSID
 #define NO_USE_SIGACTION
 #define NO_TIMES
 #define NO_GETTIMEOFDAY
-//#define NEED_PROCESS_H    although we do, this is specially handled in conf.h
+//#define NEED_PROCESS_H    although we do, this is specially handled in ap_config.h
 #define USE_LONGJMP
 #define HAVE_MMAP
 #define USE_MMAP_SCOREBOARD
@@ -107,6 +107,7 @@ API_EXPORT(int) os_spawnle(int mode,const char *cmdname,...);
  * These are used by mod_so.c
  */
 #define ap_os_dso_handle_t  HINSTANCE
+#define ap_os_dso_init()
 #define ap_os_dso_load(l)   LoadLibraryEx(l, NULL, LOAD_WITH_ALTERED_SEARCH_PATH)
 #define ap_os_dso_unload(l) FreeLibrary(l)
 #define ap_os_dso_sym(h,s)  GetProcAddress(h,s)
